@@ -14,6 +14,7 @@
  * Copyright (c) 2009      Oak Ridge National Laboratory
  * Copyright (c) 2012-2013 Los Alamos National Security, LLC.  All rights
  *                         reserved. 
+ * Copyright (c) 2013      NVIDIA Corporation.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -42,7 +43,7 @@
 #ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
 #endif
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
 #  ifdef HAVE_NETDB_H
 #  include <netdb.h>
 #  endif
@@ -68,6 +69,9 @@
 #include "btl_tcp_proc.h"
 #include "btl_tcp_frag.h"
 #include "btl_tcp_endpoint.h" 
+#if OPAL_CUDA_SUPPORT
+#include "ompi/mca/common/cuda/common_cuda.h"
+#endif /* OPAL_CUDA_SUPPORT */
 
 
 /* 
@@ -201,7 +205,7 @@ static int mca_btl_tcp_component_verify(void)
                        mca_btl_tcp_component.tcp_port_min );
         mca_btl_tcp_component.tcp_port_min = 1024;
     }
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
     if( mca_btl_tcp_component.tcp6_port_min > USHRT_MAX ) {
         opal_show_help("help-mpi-btl-tcp.txt", "invalid minimum port",
                        true, "v6", ompi_process_info.nodename,
@@ -253,7 +257,7 @@ static int mca_btl_tcp_component_register(void)
                                     (0x1 << 16) - mca_btl_tcp_component.tcp_port_min - 1,
                                     OPAL_INFO_LVL_2, &mca_btl_tcp_component.tcp_port_range);
     free(message);
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
     mca_btl_tcp_param_register_int( "port_min_v6",
                                     "The minimum port where the TCP BTL will try to bind (default 1024)", 1024,
                                     OPAL_INFO_LVL_2, & mca_btl_tcp_component.tcp6_port_min );
@@ -268,6 +272,14 @@ static int mca_btl_tcp_component_register(void)
     free(message);
 #endif
 
+    mca_btl_tcp_component.report_all_unfound_interfaces = false;
+    (void) mca_base_component_var_register(&mca_btl_tcp_component.super.btl_version,
+                                           "warn_all_unfound_interfaces",
+                                           "Issue a warning for all unfound interfaces included in if_exclude",
+                                           MCA_BASE_VAR_TYPE_BOOL,
+                                           NULL, 0, 0, OPAL_INFO_LVL_2,
+                                           MCA_BASE_VAR_SCOPE_READONLY, &mca_btl_tcp_component.report_all_unfound_interfaces);
+
     mca_btl_tcp_module.super.btl_exclusivity =  MCA_BTL_EXCLUSIVITY_LOW + 100;
     mca_btl_tcp_module.super.btl_eager_limit = 64*1024;
     mca_btl_tcp_module.super.btl_rndv_eager_limit = 64*1024;
@@ -350,7 +362,7 @@ static int mca_btl_tcp_component_open(void)
 
     /* initialize state */
     mca_btl_tcp_component.tcp_listen_sd = -1;
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
     mca_btl_tcp_component.tcp6_listen_sd = -1;
 #endif
     mca_btl_tcp_component.tcp_num_btls=0;
@@ -405,7 +417,7 @@ static int mca_btl_tcp_component_close(void)
         CLOSE_THE_SOCKET(mca_btl_tcp_component.tcp_listen_sd);
         mca_btl_tcp_component.tcp_listen_sd = -1;
     }
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
     if (mca_btl_tcp_component.tcp6_listen_sd >= 0) {
         opal_event_del(&mca_btl_tcp_component.tcp6_recv_event);
         CLOSE_THE_SOCKET(mca_btl_tcp_component.tcp6_listen_sd);
@@ -495,7 +507,7 @@ static int mca_btl_tcp_create(int if_kindex, const char* if_name)
  * (a.b.c.d/e), resolve them to an interface name (Currently only
  * supporting IPv4).  If unresolvable, warn and remove.
  */
-static char **split_and_resolve(char **orig_str, char *name)
+static char **split_and_resolve(char **orig_str, char *name, bool reqd)
 {
     int i, ret, save, if_index;
     char **argv, *str, *tmp;
@@ -568,9 +580,11 @@ static char **split_and_resolve(char **orig_str, char *name)
         
         /* If we didn't find a match, keep trying */
         if (if_index < 0) {
-            opal_show_help("help-mpi-btl-tcp.txt", "invalid if_inexclude",
-                           true, name, ompi_process_info.nodename, tmp,
-                           "Did not find interface matching this subnet");
+            if (reqd || mca_btl_tcp_component.report_all_unfound_interfaces) {
+                opal_show_help("help-mpi-btl-tcp.txt", "invalid if_inexclude",
+                               true, name, ompi_process_info.nodename, tmp,
+                               "Did not find interface matching this subnet");
+            }
             free(tmp);
             continue;
         }
@@ -674,7 +688,7 @@ static int mca_btl_tcp_component_create_instances(void)
 
     /* if the user specified an interface list - use these exclusively */
     argv = include = split_and_resolve(&mca_btl_tcp_component.tcp_if_include,
-                                       "include");
+                                       "include", true);
     while(argv && *argv) {
         char* if_name = *argv;
         int if_index = opal_ifnametokindex(if_name);
@@ -699,7 +713,7 @@ static int mca_btl_tcp_component_create_instances(void)
      * a BTL for each interface that was not excluded.
     */
     exclude = split_and_resolve(&mca_btl_tcp_component.tcp_if_exclude,
-                                "exclude");
+                                "exclude", false);
     {
         int i;
         for(i = 0; i < kif_count; i++) {
@@ -754,7 +768,7 @@ static int mca_btl_tcp_component_create_listen(uint16_t af_family)
 
     mca_btl_tcp_set_socket_options(sd);
 
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
     {
         struct addrinfo hints, *res = NULL;
         int error;
@@ -810,19 +824,19 @@ static int mca_btl_tcp_component_create_listen(uint16_t af_family)
         
         range = mca_btl_tcp_component.tcp_port_range;
         port = mca_btl_tcp_component.tcp_port_min;
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
         if (AF_INET6 == af_family) {
             range = mca_btl_tcp_component.tcp6_port_range;
             port = mca_btl_tcp_component.tcp6_port_min;
         }
-#endif  /* OPAL_WANT_IPV6 */
+#endif  /* OPAL_ENABLE_IPV6 */
 
         for( index = 0;  index < range; index++ ) {
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
             ((struct sockaddr_in6*) &inaddr)->sin6_port = htons(port + index);
 #else
             ((struct sockaddr_in*) &inaddr)->sin_port = htons(port + index);
-#endif  /* OPAL_WANT_IPV6 */
+#endif  /* OPAL_ENABLE_IPV6 */
             if(bind(sd, (struct sockaddr*)&inaddr, addrlen) < 0) {
                 if( (EADDRINUSE == opal_socket_errno) || (EADDRNOTAVAIL == opal_socket_errno) ) {
                     continue;
@@ -839,13 +853,13 @@ static int mca_btl_tcp_component_create_listen(uint16_t af_family)
                        mca_btl_tcp_component.tcp_port_min,
                        mca_btl_tcp_component.tcp_port_min + range));
         }
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
         if (AF_INET6 == af_family) {
             BTL_ERROR(("bind6() failed: no port available in the range [%d..%d]",
                        mca_btl_tcp_component.tcp6_port_min,
                        mca_btl_tcp_component.tcp6_port_min + range));
         }
-#endif  /* OPAL_WANT_IPV6 */
+#endif  /* OPAL_ENABLE_IPV6 */
         CLOSE_THE_SOCKET(sd);
         return OMPI_ERROR;
     }
@@ -862,7 +876,7 @@ static int mca_btl_tcp_component_create_listen(uint16_t af_family)
         mca_btl_tcp_component.tcp_listen_port = ((struct sockaddr_in*) &inaddr)->sin_port;
         mca_btl_tcp_component.tcp_listen_sd = sd;
     }
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
     if (AF_INET6 == af_family) {
         mca_btl_tcp_component.tcp6_listen_port = ((struct sockaddr_in6*) &inaddr)->sin6_port;
         mca_btl_tcp_component.tcp6_listen_sd = sd;
@@ -902,7 +916,7 @@ static int mca_btl_tcp_component_create_listen(uint16_t af_family)
                         0 );
         opal_event_add(&mca_btl_tcp_component.tcp_recv_event, 0);
     }
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
     if (AF_INET6 == af_family) {
         opal_event_set(opal_event_base, &mca_btl_tcp_component.tcp6_recv_event,
                         mca_btl_tcp_component.tcp6_listen_sd,
@@ -977,7 +991,7 @@ static int mca_btl_tcp_component_exchange(void)
                          opal_ifindextokindex (index);
                      current_addr++;
                  }
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
                  if ((AF_INET6 == my_ss.ss_family) &&
                      (6 != mca_btl_tcp_component.tcp_disable_family)) {
                      memcpy(&addrs[current_addr].addr_inet,
@@ -1059,7 +1073,7 @@ mca_btl_base_module_t** mca_btl_tcp_component_init(int *num_btl_modules,
     if(OMPI_SUCCESS != (ret = mca_btl_tcp_component_create_listen(AF_INET) )) {
         return 0;
     }
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
     if((ret = mca_btl_tcp_component_create_listen(AF_INET6)) != OMPI_SUCCESS) {
         if (!(OMPI_ERR_IN_ERRNO == ret &&
               EAFNOSUPPORT == opal_socket_errno)) {
@@ -1080,6 +1094,10 @@ mca_btl_base_module_t** mca_btl_tcp_component_init(int *num_btl_modules,
         return NULL;
     }
 
+#if OPAL_CUDA_SUPPORT
+    mca_common_cuda_stage_one_init();
+#endif /* OPAL_CUDA_SUPPORT */
+
     memcpy(btls, mca_btl_tcp_component.tcp_btls, mca_btl_tcp_component.tcp_num_btls*sizeof(mca_btl_tcp_module_t*));
     *num_btl_modules = mca_btl_tcp_component.tcp_num_btls;
     return btls;
@@ -1096,7 +1114,7 @@ static void mca_btl_tcp_component_accept_handler( int incoming_sd,
                                                   void* unused )
 {
     while(true) {
-#if OPAL_WANT_IPV6
+#if OPAL_ENABLE_IPV6
         struct sockaddr_in6 addr;
 #else
         struct sockaddr_in addr;
