@@ -11,7 +11,7 @@
  *                         All rights reserved.
  * Copyright (c) 2006      Sandia National Laboratories. All rights
  *                         reserved.
- * Copyright (c) 2008-2013 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2008-2014 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2012      Los Alamos National Security, LLC.  All rights
  *                         reserved. 
  * $COPYRIGHT$
@@ -55,7 +55,6 @@
 #include "opal/mca/memchecker/base/base.h"
 #include "opal/util/show_help.h"
 
-#include "ompi/mca/rte/rte.h"
 #include "ompi/constants.h"
 #include "ompi/mca/btl/btl.h"
 #include "ompi/mca/btl/base/base.h"
@@ -65,6 +64,7 @@
 #include "ompi/mca/common/verbs/common_verbs.h"
 
 #include "btl_usnic.h"
+#include "btl_usnic_connectivity.h"
 #include "btl_usnic_frag.h"
 #include "btl_usnic_endpoint.h"
 #include "btl_usnic_module.h"
@@ -74,12 +74,18 @@
 #include "btl_usnic_send.h"
 #include "btl_usnic_recv.h"
 #include "btl_usnic_proc.h"
+#include "btl_usnic_ext.h"
+#include "btl_usnic_test.h"
 
 #define OMPI_BTL_USNIC_NUM_WC       500
 #define max(a,b) ((a) > (b) ? (a) : (b))
 
+/* RNG buffer definition */
+opal_rng_buff_t ompi_btl_usnic_rand_buff;
+
 /* simulated clock */
-uint64_t usnic_ticks = 0;
+uint64_t ompi_btl_usnic_ticks = 0;
+
 static opal_event_t usnic_clock_timer_event;
 static bool usnic_clock_timer_event_set = false;
 static struct timeval usnic_clock_timeout;
@@ -94,8 +100,6 @@ static mca_btl_base_module_t **
 usnic_component_init(int* num_btl_modules, bool want_progress_threads,
                        bool want_mpi_threads);
 static int usnic_component_progress(void);
-static void seed_prng(void);
-static bool port_is_usnic(ompi_common_verbs_port_item_t *port);
 static int init_module_from_port(ompi_btl_usnic_module_t *module,
                                  ompi_common_verbs_port_item_t *port);
 
@@ -200,18 +204,27 @@ static int usnic_component_close(void)
          in their destructor */
     OBJ_DESTRUCT(&mca_btl_usnic_component.usnic_procs);
 
-    if (NULL != mca_btl_usnic_component.vendor_part_ids) {
-        free(mca_btl_usnic_component.vendor_part_ids);
-    }
-
     if (usnic_clock_timer_event_set) {
         opal_event_del(&usnic_clock_timer_event);
         usnic_clock_timer_event_set = false;
     }
 
+    /* Finalize the connectivity client and agent */
+    if (mca_btl_usnic_component.connectivity_enabled) {
+        ompi_btl_usnic_connectivity_client_finalize();
+        ompi_btl_usnic_connectivity_agent_finalize();
+    }
+
     free(mca_btl_usnic_component.usnic_all_modules);
     free(mca_btl_usnic_component.usnic_active_modules);
 
+    ompi_btl_usnic_rtnl_sk_free(mca_btl_usnic_component.unlsk);
+
+#if OMPI_BTL_USNIC_UNIT_TESTS
+    /* clean up the unit test infrastructure */
+    ompi_btl_usnic_cleanup_tests();
+#endif
+
     return OMPI_SUCCESS;
 }
 
@@ -299,6 +312,7 @@ static int check_reg_mem_basics(void)
 #endif
 }
 
+
 static int read_device_sysfs(ompi_btl_usnic_module_t *module, const char *name)
 {
     int ret, fd;
@@ -386,7 +400,7 @@ static void
 usnic_clock_callback(int fd, short flags, void *timeout)
 {
     /* 1ms == 1,000,000 ns */
-    usnic_ticks += 1000000;
+    ompi_btl_usnic_ticks += 1000000;
 
     /* run progress to make sure time change gets noticed */
     usnic_component_progress();
@@ -415,12 +429,14 @@ static mca_btl_base_module_t** usnic_component_init(int* num_btl_modules,
     usnic_if_filter_t *filter;
     bool keep_module;
     bool filter_incl = false;
-    int min_distance, num_local_procs;
+    int min_distance, num_local_procs, err;
 
     *num_btl_modules = 0;
 
     /* Currently refuse to run if MPI_THREAD_MULTIPLE is enabled */
     if (ompi_mpi_thread_multiple && !mca_btl_base_thread_multiple_override) {
+        opal_output_verbose(5, USNIC_OUT,
+                            "btl:usnic: MPI_THREAD_MULTIPLE not supported; skipping this component");
         return NULL;
     }
 
@@ -439,6 +455,14 @@ static mca_btl_base_module_t** usnic_component_init(int* num_btl_modules,
         return NULL;
     }
 
+    /* Setup the connectivity checking agent and client. */
+    if (mca_btl_usnic_component.connectivity_enabled) {
+        if (OMPI_SUCCESS != ompi_btl_usnic_connectivity_agent_init() ||
+            OMPI_SUCCESS != ompi_btl_usnic_connectivity_client_init()) {
+            return NULL;
+        }
+    }
+
     /************************************************************************
      * Below this line, we assume that usnic is loaded on all procs,
      * and therefore we will guarantee to the the modex send, even if
@@ -448,22 +472,58 @@ static mca_btl_base_module_t** usnic_component_init(int* num_btl_modules,
     /* initialization */
     mca_btl_usnic_component.my_hashed_rte_name = 
         ompi_rte_hash_name(&(ompi_proc_local()->proc_name));
+    MSGDEBUG1_OUT("%s: my_hashed_rte_name=0x%" PRIx64,
+                   __func__, mca_btl_usnic_component.my_hashed_rte_name);
 
-    seed_prng();
+    opal_srand(&ompi_btl_usnic_rand_buff, ((uint32_t) getpid()));
 
-    srandom((unsigned int)getpid());
+    err = ompi_btl_usnic_rtnl_sk_alloc(&mca_btl_usnic_component.unlsk);
+    if (0 != err) {
+        /* API returns negative errno values */
+        opal_show_help("help-mpi-btl-usnic.txt", "rtnetlink init fail",
+                       true, ompi_process_info.nodename, strerror(-err));
+        return NULL;
+    }
 
     /* Find the ports that we want to use.  We do our own interface name
      * filtering below, so don't let the verbs code see our
      * if_include/if_exclude strings */
     port_list = ompi_common_verbs_find_ports(NULL, NULL,
-                                             OMPI_COMMON_VERBS_FLAGS_UD,
+                                             OMPI_COMMON_VERBS_FLAGS_TRANSPORT_USNIC_UDP,
                                              USNIC_OUT);
-    if (NULL == port_list || 0 == opal_list_get_size(port_list)) {
-        mca_btl_base_error_no_nics("usNIC", "device");
+    if (NULL == port_list) {
+        OMPI_ERROR_LOG(ORTE_ERR_OUT_OF_RESOURCE);
         goto free_include_list;
+    } else if (opal_list_get_size(port_list) > 0) {
+        mca_btl_usnic_component.use_udp = true;
+        opal_output_verbose(5, USNIC_OUT, "btl:usnic: using UDP transport");
+    } else {
+        OBJ_RELEASE(port_list);
+
+        /* If we got no USNIC_UDP transport devices, try again with
+           USNIC */
+        port_list = ompi_common_verbs_find_ports(NULL, NULL,
+                                                 OMPI_COMMON_VERBS_FLAGS_TRANSPORT_USNIC,
+                                                 USNIC_OUT);
+
+        if (NULL == port_list) {
+            OMPI_ERROR_LOG(ORTE_ERR_OUT_OF_RESOURCE);
+            goto free_include_list;
+        } else if (opal_list_get_size(port_list) > 0) {
+            mca_btl_usnic_component.use_udp = false;
+            opal_output_verbose(5, USNIC_OUT,
+                                "btl:usnic: using L2-only transport");
+        } else {
+            mca_btl_base_error_no_nics("usNIC", "device");
+            goto free_include_list;
+        }
     }
 
+    /* Initialize the table of usnic extension function pointers */
+    item = opal_list_get_first(port_list);
+    port = (ompi_common_verbs_port_item_t*) item;
+    ompi_btl_usnic_ext_init(port->device->context);
+
     /* Setup an array of pointers to point to each module (which we'll
        return upstream) */
     mca_btl_usnic_component.num_modules = opal_list_get_size(port_list);
@@ -528,16 +588,6 @@ static mca_btl_base_module_t** usnic_component_init(int* num_btl_modules,
                             "btl:usnic: found: device %s, port %d",
                             port->device->device_name, port->port_num);
 
-        /* This component only works with Cisco VIC/usNIC devices; it
-           is not a general verbs UD component.  Reject any ports
-           found on devices that are not Cisco VICs. */
-        if (!port_is_usnic(port)) {
-            opal_output_verbose(5, USNIC_OUT,
-                                "btl:usnic: this is not a usnic-capable device");
-            --mca_btl_usnic_component.num_modules;
-            continue; /* next port */
-        }
-
         /* Fill in a bunch of the module struct */
         module = &(mca_btl_usnic_component.usnic_all_modules[i]);
         if (OMPI_SUCCESS != init_module_from_port(module, port)) {
@@ -580,6 +630,37 @@ static mca_btl_base_module_t** usnic_component_init(int* num_btl_modules,
             continue;
         }
 
+        /* Tell this device's context that we are aware that we need
+           to request the UD header length.  If it fails, just skip
+           this device. */
+        if (NULL != ompi_btl_usnic_ext.enable_udp) {
+            opal_output_verbose(5, USNIC_OUT,
+                                "btl:usnic: enabling UDP support for %s",
+                                ibv_get_device_name(module->device));
+            if (0 !=
+                ompi_btl_usnic_ext.enable_udp(port->device->context)) {
+                --mca_btl_usnic_component.num_modules;
+                opal_output_verbose(5, USNIC_OUT,
+                                    "btl:usnic: UDP support unexpectedly failed for %s; ignoring this device", 
+                                    ibv_get_device_name(module->device));
+                continue;
+            }
+
+            int len =
+                ompi_btl_usnic_ext.get_ud_header_len(port->device->context,
+                                                     port->port_num);
+            /* Sanity check: the len we get back should be 42.  If
+               it's not, skip this device. */
+            if (OMPI_BTL_USNIC_UDP_HDR_SZ != len) {
+                opal_output_verbose(5, USNIC_OUT,
+                                    "btl:usnic: unexpected UD header length for %s reported by extension (%d); ignoring this device", 
+                                    ibv_get_device_name(module->device),
+                                    len);
+                --mca_btl_usnic_component.num_modules;
+                continue;
+            }
+        }
+
         /* How many xQ entries do we want? */
         if (-1 == mca_btl_usnic_component.sd_num) {
             module->sd_num = device_attr.max_qp_wr;
@@ -626,24 +707,24 @@ static mca_btl_base_module_t** usnic_component_init(int* num_btl_modules,
         /* Find the max payload this port can handle */
         module->max_frag_payload =
             module->if_mtu - /* start with the MTU */
-            sizeof(ompi_btl_usnic_protocol_header_t) -
+            OMPI_BTL_USNIC_PROTO_HDR_SZ -
             sizeof(ompi_btl_usnic_btl_header_t); /* subtract size of
                                                     the BTL header */
         /* same, but use chunk header */
         module->max_chunk_payload =
             module->if_mtu -
-            sizeof(ompi_btl_usnic_protocol_header_t) -
+            OMPI_BTL_USNIC_PROTO_HDR_SZ -
             sizeof(ompi_btl_usnic_btl_chunk_header_t);
 
         /* Priorirty queue MTU and max size */
         if (0 == module->tiny_mtu) {
             module->tiny_mtu = 768;
             module->max_tiny_payload = module->tiny_mtu -
-                sizeof(ompi_btl_usnic_protocol_header_t) -
+                OMPI_BTL_USNIC_PROTO_HDR_SZ -
                 sizeof(ompi_btl_usnic_btl_header_t);
         } else {
             module->tiny_mtu = module->max_tiny_payload +
-                sizeof(ompi_btl_usnic_protocol_header_t) +
+                OMPI_BTL_USNIC_PROTO_HDR_SZ +
                 sizeof(ompi_btl_usnic_btl_header_t);
         }
 
@@ -725,7 +806,7 @@ static mca_btl_base_module_t** usnic_component_init(int* num_btl_modules,
                             module->rd_num,
                             module->cq_num);
         opal_output_verbose(5, USNIC_OUT,
-                            "btl:usnic: priority MTU %s:%d = %d",
+                            "btl:usnic: priority MTU %s:%d = %" PRIsize_t,
                             ibv_get_device_name(module->device),
                             module->port_num,
                             module->tiny_mtu);
@@ -868,7 +949,7 @@ static int usnic_component_progress(void)
     static bool fastpath_ok=true;
 
     /* update our simulated clock */
-    usnic_ticks += 5000;
+    ompi_btl_usnic_ticks += 5000;
 
     count = 0;
     if (fastpath_ok) {
@@ -882,7 +963,8 @@ static int usnic_component_progress(void)
                 if (OPAL_LIKELY(wc.opcode == IBV_WC_RECV &&
                                 wc.status == IBV_WC_SUCCESS)) {
                     rseg = (ompi_btl_usnic_recv_segment_t*)(intptr_t)wc.wr_id;
-                    ompi_btl_usnic_recv_fast(module, rseg, channel);
+                    ompi_btl_usnic_recv_fast(module, rseg, channel,
+                                             wc.byte_len);
                     fastpath_ok = false;    /* prevent starvation */
                     return 1;
                 } else {
@@ -913,7 +995,7 @@ static int usnic_handle_completion(
            going.  The sender will eventually re-send it. */
         if (IBV_WC_RECV == cwc->opcode) {
             if (cwc->byte_len <
-                 (sizeof(ompi_btl_usnic_protocol_header_t)+
+                 (OMPI_BTL_USNIC_PROTO_HDR_SZ +
                   sizeof(ompi_btl_usnic_btl_header_t))) {
                 BTL_ERROR(("%s: RX error polling CQ[%d] with status %d for wr_id %" PRIx64 " vend_err %d, byte_len %d",
                    ibv_get_device_name(module->device),
@@ -965,7 +1047,7 @@ static int usnic_handle_completion(
     /**** Receive completions ****/
     case OMPI_BTL_USNIC_SEG_RECV:
         assert(IBV_WC_RECV == cwc->opcode);
-        ompi_btl_usnic_recv(module, rseg, channel);
+        ompi_btl_usnic_recv(module, rseg, channel, cwc->byte_len);
         break;
 
     default:
@@ -988,7 +1070,7 @@ static int usnic_component_progress_2(void)
     int c;
 
     /* update our simulated clock */
-    usnic_ticks += 5000;
+    ompi_btl_usnic_ticks += 5000;
 
     /* Poll for completions */
     for (i = 0; i < mca_btl_usnic_component.num_modules; i++) {
@@ -1046,43 +1128,6 @@ static int usnic_component_progress_2(void)
     return count;
 }
 
-static void seed_prng(void)
-{
-    unsigned short seedv[3];
-    seedv[0] = OMPI_PROC_MY_NAME->vpid;
-    seedv[1] = opal_timer_base_get_cycles();
-    usleep(1);
-    seedv[2] = opal_timer_base_get_cycles();
-    seed48(seedv);
-}
-
-static bool port_is_usnic(ompi_common_verbs_port_item_t *port)
-{
-    bool is_usnic = false;
-    uint32_t *vpi;
-
-#if BTL_USNIC_HAVE_IBV_USNIC
-    /* If we have the IB_*_USNIC constants, then take any
-       device which advertises them */
-    if (IBV_TRANSPORT_USNIC == port->device->device->transport_type &&
-        IBV_NODE_USNIC == port->device->device->node_type) {
-        is_usnic = true;
-    }
-#endif
-    /* Or take any specific device that we know is a Cisco
-       VIC.  Cisco's vendor ID is 0x1137. */
-    if (!is_usnic && 0x1137 == port->device->device_attr.vendor_id) {
-        for (vpi = mca_btl_usnic_component.vendor_part_ids; *vpi > 0; ++vpi) {
-            if (port->device->device_attr.vendor_part_id == *vpi) {
-                is_usnic = true;
-                break;
-            }
-        }
-    }
-
-    return is_usnic;
-}
-
 /* returns OMPI_SUCCESS if module initialization was successful, OMPI_ERROR
  * otherwise */
 static int init_module_from_port(ompi_btl_usnic_module_t *module,
@@ -1160,6 +1205,7 @@ static int init_module_from_port(ompi_btl_usnic_module_t *module,
             return OMPI_ERROR;
         }
     }
+    module->local_addr.link_speed_mbps = module->super.btl_bandwidth;
     opal_output_verbose(5, USNIC_OUT,
                         "btl:usnic: bandwidth for %s:%d = %u",
                         ibv_get_device_name(module->device),
@@ -1466,7 +1512,7 @@ static void dump_endpoint(ompi_btl_usnic_endpoint_t *endpoint)
         }
     }
 
-    opal_output(0, "      ack_needed=%s n_t=%"PRIu64" n_a=%"PRIu64" n_r=%"PRIu64" n_s=%"PRIu64" rfstart=%"PRIu32"\n",
+    opal_output(0, "      ack_needed=%s n_t=%"UDSEQ" n_a=%"UDSEQ" n_r=%"UDSEQ" n_s=%"UDSEQ" rfstart=%"PRIu32"\n",
                 (endpoint->endpoint_ack_needed?"true":"false"),
                 endpoint->endpoint_next_seq_to_send,
                 endpoint->endpoint_ack_seq_rcvd,
@@ -1532,3 +1578,5 @@ void ompi_btl_usnic_component_debug(void)
         ompi_btl_usnic_print_stats(module, "  manual", /*reset=*/false);
     }
 }
+
+#include "test/btl_usnic_component_test.h"
