@@ -1,6 +1,5 @@
 /**
   Copyright (c) 2011 Mellanox Technologies. All rights reserved.
-  Copyright (c) 2012 Cisco Systems, Inc.  All rights reserved.
   $COPYRIGHT$
 
   Additional copyrights may follow
@@ -13,6 +12,20 @@
  * Initial query function that is invoked during MPI_INIT, allowing
  * this module to indicate what level of thread support it provides.
  */
+
+int modular_pow(uint64_t base, uint64_t exponent, uint64_t modulus)
+{
+    int result = 1;
+    while(exponent>0)
+    {
+        if((exponent % 2) == 1)
+            result = (result * base) % modulus;
+        exponent = exponent >> 1;
+        base = (base * base) % modulus;
+    }
+    return result;
+}
+
 int mca_coll_fca_init_query(bool enable_progress_threads,
                             bool enable_mpi_threads)
 {
@@ -89,21 +102,25 @@ static int __get_local_ranks(mca_coll_fca_module_t *fca_module)
 static int __fca_comm_new(mca_coll_fca_module_t *fca_module)
 {
     ompi_communicator_t *comm = fca_module->comm;
-    fca_comm_new_spec_t spec;
+    fca_comm_new_spec_t spec = {0,};
     int info_size, all_info_size;
-    void *all_info=NULL, *my_info=NULL;
-    int *rcounts=NULL, *disps=NULL;
-    int i, rc, ret;
+    void *all_info, *my_info;
+    int *rcounts, *displs;
+    int i, rc, ret, comm_size = ompi_comm_size(fca_module->comm);
 
     /* call fca_get_rank_info() on node managers only*/
     if (fca_module->local_proc_idx == 0) {
+#if OMPI_FCA_VERSION >= 30
         my_info = fca_get_rank_info(mca_coll_fca_component.fca_context,
-                                                               &info_size);
+                                    fca_module->rank, &info_size);
+#else
+        my_info = fca_get_rank_info(mca_coll_fca_component.fca_context,
+                                    &info_size);
+#endif
         if (!my_info) {
             FCA_ERROR("fca_get_rank_info returned NULL");
             return OMPI_ERROR;
         }
-
     } else {
         info_size = 0;
     }
@@ -111,7 +128,7 @@ static int __fca_comm_new(mca_coll_fca_module_t *fca_module)
 
     /* Allocate gather buffer on the root rank */
     if (fca_module->rank == 0) {
-        rcounts = calloc(ompi_comm_size(comm), sizeof *rcounts);
+        rcounts = calloc(comm_size, sizeof *rcounts);
     }
 
     /* Get all rank info sizes using MPI_Gather */
@@ -123,18 +140,26 @@ static int __fca_comm_new(mca_coll_fca_module_t *fca_module)
     /* Allocate buffer for gathering rank information on rank0 */
     if (fca_module->rank == 0) {
         all_info_size = 0;
-        FCA_MODULE_VERBOSE(fca_module, 1, "Total rank_info size: %d", all_info_size);
-        disps = calloc(ompi_comm_size(comm), sizeof *disps);
-        for (i = 0; i < ompi_comm_size(comm); ++i) {
-            disps[i] = all_info_size;
+        displs = calloc(comm_size, sizeof *displs);
+
+        for (i = 0; i < comm_size; ++i) {
+            displs[i] = all_info_size;
             all_info_size += rcounts[i];
+
+            if (rcounts[i] > 0)
+                ++spec.rank_count;
+
+            FCA_MODULE_VERBOSE(fca_module, 1, "gatherv: rcounts[%d]=%d displs[%d]=%d",
+                               i, rcounts[i], i, displs[i]);
         }
+
+        FCA_MODULE_VERBOSE(fca_module, 1, "Total rank_info size: %d", all_info_size);
         all_info = calloc(all_info_size, 1);
     }
 
-    /* Send all node managers information to rank0 using MPI_Gatherv*/
+    /* Send all node managers information to rank0 using MPI_Gatherv */
     rc = comm->c_coll.coll_gatherv(my_info, info_size, MPI_BYTE,
-                                   all_info, rcounts, disps, MPI_BYTE, 0,
+                                   all_info, rcounts, displs, MPI_BYTE, 0,
                                    comm, comm->c_coll.coll_gather_module);
     if (rc != OMPI_SUCCESS) {
         FCA_ERROR("Failed to gather rank information to rank0: %d", rc);
@@ -145,21 +170,21 @@ static int __fca_comm_new(mca_coll_fca_module_t *fca_module)
     if (fca_module->rank == 0) {
         spec.rank_info  = all_info;
         spec.is_comm_world = comm == MPI_COMM_WORLD;
-        spec.rank_count = 0;
-        for (i = 0; i < ompi_comm_size(comm); ++i) {
-            FCA_MODULE_VERBOSE(fca_module, 1, "rcounts[%d]=%d disps[%d]=%d",
-                               i, rcounts[i], i, disps[i]);
-            if (rcounts[i] > 0)
-                ++spec.rank_count;
-        }
-        free(disps);
-        free(rcounts);
 
+        free(displs);
+        free(rcounts);
+#if OMPI_FCA_VERSION >= 30
+        spec.comm_size = comm_size;
+
+        spec.comm_init_data = NULL;
+        spec.comm_init_data_size = 0;
+#endif
         FCA_MODULE_VERBOSE(fca_module, 1, "starting fca_comm_new(), rank_count: %d",
                            spec.rank_count);
 
         ret = fca_comm_new(mca_coll_fca_component.fca_context,
-                                                      &spec, &fca_module->fca_comm_desc);
+                           &spec, &fca_module->fca_comm_desc);
+
         free(all_info);
     }
 
@@ -176,7 +201,30 @@ static int __fca_comm_new(mca_coll_fca_module_t *fca_module)
         FCA_ERROR("COMM_NEW failed: %s", fca_strerror(ret));
         return OMPI_ERROR;
     }
+#if OMPI_FCA_VERSION >= 30
+    /* Send comm_ini_data_size to all ranks in comm */
+    rc = fca_module->previous_bcast(&spec.comm_init_data_size, 1, MPI_INT,
+                                    0, comm, fca_module->previous_bcast_module);
+    if (OMPI_SUCCESS != rc) {
+        FCA_ERROR("Failed to broadcast comm init data size value from rank0: %d", rc);
+        return rc;
+    }
 
+    if (0 != fca_module->rank &&
+        NULL == (spec.comm_init_data = calloc(1, spec.comm_init_data_size))) {
+        FCA_ERROR("Failed to allocate memory for comm init data.");
+        return OMPI_ERROR;
+    }
+
+    /* Send comm_ini_data to all ranks in comm */
+    rc = fca_module->previous_scatter(spec.comm_init_data, spec.comm_init_data_size, MPI_BYTE,
+                                      spec.comm_init_data, spec.comm_init_data_size, MPI_BYTE,
+                                      0, comm, fca_module->previous_scatter_module);
+    if (OMPI_SUCCESS != rc) {
+        FCA_ERROR("Failed to scatter comm_init sizes return value from rank0: %d", rc);
+        return rc;
+    }
+#endif
     /* Release allocate rank_info on node managers */
     if (fca_module->local_proc_idx == 0) {
         fca_free_rank_info(my_info);
@@ -193,36 +241,173 @@ static int __fca_comm_new(mca_coll_fca_module_t *fca_module)
 
     FCA_MODULE_VERBOSE(fca_module, 1, "Received FCA communicator spec, comm_id %d",
                        fca_module->fca_comm_desc.comm_id);
-    return OMPI_SUCCESS;
-}
-
-static int __create_fca_comm(mca_coll_fca_module_t *fca_module)
-{
-    int comm_size;
-    int rc, ret;
-
-    rc = __fca_comm_new(fca_module);
-    if (rc != OMPI_SUCCESS)
-        return rc;
 
+#if OMPI_FCA_VERSION >= 30
     /* allocate comm_init_spec */
     FCA_MODULE_VERBOSE(fca_module, 1, "Starting COMM_INIT comm_id %d proc_idx %d num_procs %d",
                        fca_module->fca_comm_desc.comm_id, fca_module->local_proc_idx,
                        fca_module->num_local_procs);
 
-    comm_size = ompi_comm_size(fca_module->comm);
     ret = mca_coll_fca_comm_init(mca_coll_fca_component.fca_context,
                                  fca_module->rank, comm_size,
                                  fca_module->local_proc_idx, fca_module->num_local_procs,
-                                 &fca_module->fca_comm_desc, &fca_module->fca_comm);
+                                 &fca_module->fca_comm_desc, &fca_module->fca_comm,
+                                 spec.comm_init_data);
     if (ret < 0) {
         FCA_ERROR("COMM_INIT failed: %s", fca_strerror(ret));
         return OMPI_ERROR;
     }
 
+    if (0 != fca_module->rank) {
+        free(spec.comm_init_data);
+    }
+#endif
+    return OMPI_SUCCESS;
+}
+
+static int __create_fca_comm(mca_coll_fca_module_t *fca_module)
+{
+    int rc, ret;
+    int result = MPI_UNEQUAL;
+    mca_coll_fca_c_cache_item_t *c_item = NULL, *c_item_new = NULL;
+    mca_coll_fca_component_t *cm = &mca_coll_fca_component;
+    int comm_size = ompi_comm_size(fca_module->comm);
+    ompi_communicator_t *comm = fca_module->comm;
+    opal_list_t *c_cache;
+    struct timeval start, end, seq_start, seq_end, par_start, par_end;
+    int act_deep;
+    
+
+    if(mca_coll_fca_component.fca_verbose == 10) {
+        gettimeofday(&start, NULL);
+    }
+
+    if(mca_coll_fca_component.fca_enable_cache) {
+
+        c_cache = &cm->c_cache;
+
+        if(mca_coll_fca_component.fca_enable_hash){
+
+            int  grank = ORTE_PROC_MY_NAME->vpid;
+            int hash_index, part_of_hash_index;
+
+            if(mca_coll_fca_component.fca_parallel_hash_calc == 1) {
+                
+                if(mca_coll_fca_component.fca_verbose == 10){
+                    gettimeofday(&par_start, NULL);
+                }
+
+                part_of_hash_index = modular_pow(cm->fca_primes[grank % cm->fca_number_of_primes], grank, cm->fca_hash_size);
+                rc = comm->c_coll.coll_allreduce(&part_of_hash_index, &hash_index, 1, MPI_INT, MPI_SUM, comm, comm->c_coll.coll_allreduce_module);
+                if (rc != OMPI_SUCCESS) {
+                    FCA_ERROR("Failed to reduce hash_index: %d", rc);
+                    return rc;
+                }
+
+                if(mca_coll_fca_component.fca_verbose == 10){
+                    gettimeofday(&par_end, NULL);
+                    mca_coll_fca_component.fca_work_time_parallel =+
+                        par_end.tv_sec - par_start.tv_sec + 1e-6 * (par_end.tv_usec - par_start.tv_usec);
+                }
+            }else{
+
+                if(mca_coll_fca_component.fca_verbose == 10){
+                    gettimeofday(&seq_start, NULL);
+                }
+
+                hash_index = 0;
+                int q_counter = 0;
+                int q_comm_size = ompi_comm_size (comm);
+
+                for(q_counter = 0; q_counter < q_comm_size; q_counter++)
+                {
+                    hash_index += modular_pow(cm->fca_primes[q_counter % cm->fca_number_of_primes], q_counter, cm->fca_hash_size);
+                }
+
+                if(mca_coll_fca_component.fca_verbose == 10){
+                    gettimeofday(&seq_end, NULL);
+                    mca_coll_fca_component.fca_work_time_sequency =+
+                        seq_end.tv_sec - seq_start.tv_sec + 1e-6 * (seq_end.tv_usec - seq_start.tv_usec);
+                }
+
+            }
+
+            if(cm->fca_hash[hash_index % cm->fca_hash_size] != NULL)
+            {
+                c_cache = cm->fca_hash[hash_index % cm->fca_hash_size];
+                if(mca_coll_fca_component.fca_verbose == 10) {
+                    gettimeofday(&end, NULL);
+                    mca_coll_fca_component.fca_total_work_time =+
+                        end.tv_sec - start.tv_sec + 1e-6 * (end.tv_usec - start.tv_usec);
+                    mca_coll_fca_component.fca_hash_hit += 1;
+                }
+            }else
+            {
+                if(mca_coll_fca_component.fca_verbose == 10) {
+                    mca_coll_fca_component.fca_hash_miss += 1;
+                }
+                c_cache = OBJ_NEW(opal_list_t);
+                cm->fca_hash[hash_index % cm->fca_hash_size] = c_cache;
+            }
+
+        }
+
+        act_deep = 0;
+        for( c_item = (mca_coll_fca_c_cache_item_t *)opal_list_get_first(c_cache);
+                c_item != (mca_coll_fca_c_cache_item_t *)opal_list_get_end(c_cache);
+                c_item = (mca_coll_fca_c_cache_item_t *)opal_list_get_next((opal_list_item_t *) c_item)){
+            act_deep++;
+            /* first check the size */
+            if( comm_size == c_item->size){
+                /* then we have a potential cache hit */
+                ompi_comm_compare(comm, c_item->comm, &result); 
+                if( MPI_CONGRUENT == result) {
+                    /* cache hit! Return the context and be done with it */
+                    /* first bump the score */
+                    ret = fca_comm_get_caps(c_item->fca_comm_wrap->fca_comm, &fca_module->fca_comm_caps);
+                    if (ret < 0) {
+                        FCA_ERROR("GET_COMM_CAPS failed: %s", fca_strerror(ret));
+                        return OMPI_ERROR;
+                    }
+                    fca_module->fca_comm = c_item->fca_comm_wrap->fca_comm;
+
+                    if(mca_coll_fca_component.fca_verbose == 10) {
+                        gettimeofday(&end, NULL);
+                        
+                        mca_coll_fca_component.fca_total_work_time =+
+                            end.tv_sec - start.tv_sec + 1e-6 * (end.tv_usec - start.tv_usec);
+                        
+                        mca_coll_fca_component.fca_cache_hit += 1;
+                        
+                        if(act_deep>mca_coll_fca_component.fca_max_deep_in_cache)
+                            mca_coll_fca_component.fca_max_deep_in_cache = act_deep;
+                    }
+                    return OMPI_SUCCESS;
+                }
+            }
+        }
+    }
+    rc = __fca_comm_new(fca_module);
+    if (OMPI_SUCCESS != rc) {
+        return rc;
+    }
+#if OMPI_FCA_VERSION < 30
+    /* allocate comm_init_spec */
+    FCA_MODULE_VERBOSE(fca_module, 1, "Starting COMM_INIT comm_id %d proc_idx %d num_procs %d",
+                       fca_module->fca_comm_desc.comm_id, fca_module->local_proc_idx,
+                       fca_module->num_local_procs);
+
+    ret = mca_coll_fca_comm_init(mca_coll_fca_component.fca_context,
+                                 fca_module->rank, ompi_comm_size(fca_module->comm),
+                                 fca_module->local_proc_idx, fca_module->num_local_procs,
+                                 &fca_module->fca_comm_desc, &fca_module->fca_comm);
+    if (ret < 0) {
+        FCA_ERROR("COMM_INIT failed: %s", fca_strerror(ret));
+        return OMPI_ERROR;
+     }
+#endif
     /* get communicator capabilities */
-    ret = fca_comm_get_caps(fca_module->fca_comm,
-                                                       &fca_module->fca_comm_caps);
+    ret = fca_comm_get_caps(fca_module->fca_comm, &fca_module->fca_comm_caps);
     if (ret < 0) {
         FCA_ERROR("GET_COMM_CAPS failed: %s", fca_strerror(ret));
         return OMPI_ERROR;
@@ -230,7 +415,32 @@ static int __create_fca_comm(mca_coll_fca_module_t *fca_module)
 
     /* by this point every rank in the communicator is set up */
     FCA_MODULE_VERBOSE(fca_module, 1, "Initialized FCA communicator, comm_id %d",
-                       fca_module->fca_comm_desc.comm_id);
+            fca_module->fca_comm_desc.comm_id);
+    if(mca_coll_fca_component.fca_enable_cache) {
+
+        c_item_new = OBJ_NEW(mca_coll_fca_c_cache_item_t);
+        c_item_new->fca_comm_wrap = OBJ_NEW(mca_coll_fca_comm_wrap_t);
+
+        OBJ_RETAIN(comm); 
+
+        c_item_new->size = comm_size;
+        c_item_new->comm = comm;
+        c_item_new->fca_comm_wrap->fca_comm = fca_module->fca_comm;
+        c_item_new->fca_comm_wrap->rank = fca_module->rank;
+        c_item_new->fca_comm_wrap->comm_id = fca_module->fca_comm_desc.comm_id;
+
+        opal_list_append(c_cache,(opal_list_item_t *) c_item_new);
+    }
+
+    if(mca_coll_fca_component.fca_verbose == 10) {
+        
+        gettimeofday(&end, NULL);
+        
+        mca_coll_fca_component.fca_total_work_time =+
+            end.tv_sec - start.tv_sec + 1e-6 * (end.tv_usec - start.tv_usec);
+
+        mca_coll_fca_component.fca_cache_miss += 1;
+    }
     return OMPI_SUCCESS;
 }
 
@@ -255,7 +465,7 @@ static void __destroy_fca_comm(mca_coll_fca_module_t *fca_module)
     fca_module->previous_ ## __api            = comm->c_coll.coll_ ## __api;\
     fca_module->previous_ ## __api ## _module = comm->c_coll.coll_ ## __api ## _module;\
     if (!comm->c_coll.coll_ ## __api || !comm->c_coll.coll_ ## __api ## _module) {\
-	    FCA_VERBOSE(1, "(%d/%s): no underlying " # __api"; disqualifying myself",\
+        FCA_VERBOSE(1, "(%d/%s): no underlying " # __api"; disqualifying myself",\
                     comm->c_contextid, comm->c_name);\
         return OMPI_ERROR;\
     }\
@@ -278,6 +488,9 @@ static int __save_coll_handlers(mca_coll_fca_module_t *fca_module)
     FCA_SAVE_PREV_COLL_API(alltoallv);
     FCA_SAVE_PREV_COLL_API(alltoallw);
     FCA_SAVE_PREV_COLL_API(reduce_scatter);
+#if OMPI_FCA_VERSION >= 30
+    FCA_SAVE_PREV_COLL_API(scatter);
+#endif
 
     return OMPI_SUCCESS;
 }
@@ -350,6 +563,13 @@ static void mca_coll_fca_module_construct(mca_coll_fca_module_t *fca_module)
 static void mca_coll_fca_module_destruct(mca_coll_fca_module_t *fca_module)
 {
     FCA_VERBOSE(5, "==>");
+    if(mca_coll_fca_component.fca_enable_cache == 0) {
+
+        if (fca_module->fca_comm) {
+            __destroy_fca_comm(fca_module);
+        } 
+    }
+
     OBJ_RELEASE(fca_module->previous_barrier_module);
     OBJ_RELEASE(fca_module->previous_bcast_module);
     OBJ_RELEASE(fca_module->previous_reduce_module);
@@ -362,8 +582,10 @@ static void mca_coll_fca_module_destruct(mca_coll_fca_module_t *fca_module)
     OBJ_RELEASE(fca_module->previous_alltoallv_module);
     OBJ_RELEASE(fca_module->previous_alltoallw_module);
     OBJ_RELEASE(fca_module->previous_reduce_scatter_module);
-    if (fca_module->fca_comm)
-        __destroy_fca_comm(fca_module);
+#if OMPI_FCA_VERSION >= 30
+    OBJ_RELEASE(fca_module->previous_scatter_module);
+#endif
+
     free(fca_module->local_ranks);
     mca_coll_fca_module_clear(fca_module);
 }
@@ -431,3 +653,54 @@ OBJ_CLASS_INSTANCE(mca_coll_fca_module_t,
                    mca_coll_base_module_t,
                    mca_coll_fca_module_construct,
                    mca_coll_fca_module_destruct);
+
+
+static void mca_coll_fca_comm_wrap_constructor(mca_coll_fca_comm_wrap_t *item) {
+    item->fca_comm = NULL;
+    item->comm_id = -1;
+    item->rank = -1;
+}
+
+static void mca_coll_fca_comm_wrap_destruct(mca_coll_fca_comm_wrap_t *item) {
+
+    int ret;
+
+    if(item->fca_comm != NULL)
+    {
+        fca_comm_destroy(item->fca_comm);
+        if (item->rank == 0) {
+            ret = fca_comm_end(mca_coll_fca_component.fca_context,
+                    item->comm_id);
+            if (ret < 0) {
+                FCA_ERROR("COMM_END failed: %s", fca_strerror(ret));
+            }
+        }
+
+    }
+
+}
+
+OBJ_CLASS_INSTANCE(mca_coll_fca_comm_wrap_t,
+        opal_object_t,
+        mca_coll_fca_comm_wrap_constructor,
+        mca_coll_fca_comm_wrap_destruct);
+
+
+static void mca_coll_fca_c_cache_item_construct(mca_coll_fca_c_cache_item_t *item) {
+        item->comm = NULL;
+        item->size = -1;
+        /* item->fca_comm_wrap = OBJ_NEW(mca_coll_fca_comm_wrap_t); */
+        item->fca_comm_wrap = NULL;
+}
+
+static void mca_coll_fca_c_cache_item_destruct(mca_coll_fca_c_cache_item_t *item) {
+    if(item->fca_comm_wrap != NULL) {
+        OBJ_RELEASE(item->fca_comm_wrap);
+        /* OBJ_RELEASE(item->comm); */
+    }
+} 
+
+OBJ_CLASS_INSTANCE(mca_coll_fca_c_cache_item_t,
+        opal_list_item_t,
+        mca_coll_fca_c_cache_item_construct,
+        mca_coll_fca_c_cache_item_destruct);
