@@ -22,6 +22,7 @@
 #include "coll_libnbc.h"
 #include "ompi/include/ompi/constants.h"
 #include "ompi/request/request.h"
+#include "ompi/datatype/ompi_datatype.h"
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -479,14 +480,14 @@ static inline int NBC_Type_intrinsic(MPI_Datatype type) {
 /* let's give a try to inline functions */
 static inline int NBC_Copy(void *src, int srccount, MPI_Datatype srctype, void *tgt, int tgtcount, MPI_Datatype tgttype, MPI_Comm comm) {
   int size, pos, res;
-  MPI_Aint ext;
+  OPAL_PTRDIFF_TYPE ext, lb;
   void *packbuf;
 
   if((srctype == tgttype) && NBC_Type_intrinsic(srctype)) {
     /* if we have the same types and they are contiguous (intrinsic
      * types are contiguous), we can just use a single memcpy */
-    res = MPI_Type_extent(srctype, &ext);
-    if (MPI_SUCCESS != res) { printf("MPI Error in MPI_Type_extent() (%i)\n", res); return res; }
+    res = ompi_datatype_get_extent(srctype, &lb, &ext);
+    if (OMPI_SUCCESS != res) { printf("MPI Error in MPI_Type_extent() (%i)\n", res); return res; }
     memcpy(tgt, src, srccount*ext);
   } else {
     /* we have to pack and unpack */
@@ -508,14 +509,14 @@ static inline int NBC_Copy(void *src, int srccount, MPI_Datatype srctype, void *
 
 static inline int NBC_Unpack(void *src, int srccount, MPI_Datatype srctype, void *tgt, MPI_Comm comm) {
   int size, pos, res;
-  MPI_Aint ext;
+  OPAL_PTRDIFF_TYPE ext, lb;
 
   if(NBC_Type_intrinsic(srctype)) {
     /* if we have the same types and they are contiguous (intrinsic
      * types are contiguous), we can just use a single memcpy */
-    res = MPI_Type_extent(srctype, &ext);
-    if (MPI_SUCCESS != res) { printf("MPI Error in MPI_Type_extent() (%i)\n", res); return res; }
-    memcpy(tgt, src, srccount*ext);
+    res = ompi_datatype_get_extent (srctype, &lb, &ext);
+    if (OMPI_SUCCESS != res) { printf("MPI Error in MPI_Type_extent() (%i)\n", res); return res; }
+    memcpy(tgt, src, srccount * ext);
 
   } else {
     /* we have to unpack */
@@ -557,6 +558,9 @@ static inline void NBC_SchedCache_dictwipe(hb_tree *dict, int *size) {
   } \
 }
 
+int NBC_Comm_neighbors_count(MPI_Comm comm, int *indegree, int *outdegree, int *weighted);
+int NBC_Comm_neighbors(MPI_Comm comm, int maxindegree, int sources[], int sourceweights[], int maxoutdegree, int destinations[], int destweights[]);
+
 #ifdef __cplusplus
 }
 #endif
