@@ -75,8 +75,8 @@ int ml_coll_up_and_down_hier_setup(mca_coll_ml_module_t *ml_module,
                 collective_alg->n_functions ));
 
     /* allocate space for the functions */
-    collective_alg->functions = (coll_ml_function_t *)
-        malloc(sizeof(coll_ml_function_t) * collective_alg->n_functions);
+    collective_alg->functions = (mca_bcol_base_function_t *)
+        malloc(sizeof(mca_bcol_base_function_t) * collective_alg->n_functions);
     if( NULL == collective_alg->functions) {
         ML_ERROR(("Can't allocate memory.\n"));
         ret = OMPI_ERR_OUT_OF_RESOURCE;
@@ -300,7 +300,10 @@ int ml_coll_hier_allreduce_setup(mca_coll_ml_module_t *ml_module)
     return ret;
 }
 
-
+#if 0
+/*
+ * Manju: New setup function in coll_ml_hier_algorithms_reduce_setup.c
+ */
 /* Ishai: Reduce is not an hier algorithm (it is rooted) - it needs a different ML algorithm */
 /* Need to rewrite */
 int ml_coll_hier_reduce_setup(mca_coll_ml_module_t *ml_module)
@@ -320,6 +323,7 @@ int ml_coll_hier_reduce_setup(mca_coll_ml_module_t *ml_module)
     ml_module->topo_list[topo_index].hierarchical_algorithms[BCOL_BCAST] = NULL;
     return ret;
 }
+#endif
 
 int ml_coll_barrier_constant_group_data_setup(
                 mca_coll_ml_topology_t *topo_info,
