@@ -9,7 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2006-2012 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2006-2014 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2006-2012 Mellanox Technologies.  All rights reserved.
  * Copyright (c) 2006-2007 Los Alamos National Security, LLC.  All rights
  *                         reserved.
@@ -163,10 +163,16 @@ static bool want_this_port(char **include_list, char **exclude_list,
 static const char *transport_name_to_str(enum ibv_transport_type transport_type)
 {
     switch(transport_type) {
-    case IBV_TRANSPORT_IB:      return "IB";
-    case IBV_TRANSPORT_IWARP:   return "IWARP";
+    case IBV_TRANSPORT_IB:        return "IB";
+    case IBV_TRANSPORT_IWARP:     return "IWARP";
+#if HAVE_DECL_IBV_TRANSPORT_USNIC
+    case IBV_TRANSPORT_USNIC:     return "usNIC";
+#endif
+#if HAVE_DECL_IBV_TRANSPORT_USNIC_UDP
+    case IBV_TRANSPORT_USNIC_UDP: return "usNIC UDP";
+#endif
     case IBV_TRANSPORT_UNKNOWN: 
-    default:                    return "unknown";
+    default:                      return "unknown";
     }
 }
 
@@ -182,6 +188,139 @@ static const char *link_layer_to_str(int link_type)
 }
 #endif
 
+/* Helper routine to detect Cisco usNIC devices (these are non-IB, non-RoCE,
+ * non-iWARP devices).  See the usnic BTL for more information.
+ *
+ * Once usNIC is no longer new and the IBV_TRANSPORT_USNIC constant is widely
+ * available in the wild, all calls to it can be replaced with a simple check
+ * against said constant.
+ */
+static bool device_is_usnic(struct ibv_device *device)
+{
+    /* A usNIC-capable VIC will present as one of:
+     *   1. _IWARP -- any libibverbs and old kernel
+     *   2. _UNKNOWN -- old libibverbs and new kernel
+     *   3. _USNIC -- new libibverbs and new kernel
+     *
+     * Where an "old kernel" is one that does not have this commit:
+     * http://bit.ly/kernel-180771a3
+     */
+#if HAVE_DECL_IBV_TRANSPORT_USNIC
+    if (IBV_TRANSPORT_USNIC == device->transport_type) {
+        return true;
+    }
+#endif
+#if HAVE_DECL_IBV_TRANSPORT_USNIC_UDP
+    if (IBV_TRANSPORT_USNIC_UDP == device->transport_type) {
+        return true;
+    }
+#endif
+    if ((IBV_TRANSPORT_IWARP == device->transport_type ||
+         IBV_TRANSPORT_UNKNOWN == device->transport_type) &&
+        0 == strncmp(device->name, "usnic_", strlen("usnic_"))) {
+        /* if we are willing to open the device, query its attributes, then
+         * close it again, we could also check for Cisco's vendor ID (0x1137) */
+        return true;
+    }
+
+    return false;
+}
+
+enum {
+    USNIC_L2,
+    USNIC_UDP,
+    USNIC_UNKNOWN
+};
+
+/* See comment in btl_usnic_ext.c about why we must check the return
+   from the usnic verbs extensions probe for a magic number (which
+   means we must also copy the usnic extension struct and magic number
+   value down here into common/verbs.  Bummer). */
+typedef union {
+    struct {
+        int lookup_version;
+        uint64_t magic;
+    } qpt;
+    struct ibv_port_attr attr;
+} port_query_u;
+
+#define USNIC_PORT_QUERY_MAGIC (0x43494e7375534355ULL)
+
+/*
+ * usNIC devices will always return one of three
+ * device->transport_type values:
+ *
+ * 1. TRANSPORT_IWARP: for older kernels (e.g., on systems such as
+ * RHEL 6.x (x>=4) with the drivers downloaded from cisco.com) where
+ * the cisco.com drivers could not modify verbs.h to include
+ * TRANSPORT_USNIC*.  In this case, it is unknown whether the
+ * transport is usNIC/L2 or usNIC/UDP -- you have to do an additional
+ * probe to figure it out.
+ *
+ * 2. TRANSPORT_USNIC: should probably never be returned on a customer
+ * system.  In this case, the transport is guaranteed to be usNIC/L2.
+ *
+ * 3. TRANSPORT_USNIC_UDP: on systems with new kernels and new
+ * libibverbs.  In this case, the transport is guaranteed to be
+ * usNIC/UDP.
+*/
+static int usnic_transport(struct ibv_device *device,
+                           struct ibv_context *context)
+{
+    if (!device_is_usnic(device)) {
+        return USNIC_UNKNOWN;
+    }
+
+    /* If we got a transport type of IWARP, see if the magic query
+       exists.  If it does, the transport type is UDP.  If it does
+       not, the transport type is L2.
+
+       NB: it will be complicated if we ever need to extend this
+       scheme (e.g., if we support something other than UDP someday),
+       because the real way to know what the actual transport is will
+       be to call a usnic verbs extension, and that code is all
+       currently over in the usnic BTL, which we can't call from
+       here. */
+    if (IBV_TRANSPORT_IWARP == device->transport_type) {
+        int rc;
+        port_query_u u;
+        rc = ibv_query_port(context, 42, &u.attr);
+        /* See comment in btl_usnic_ext.c about why we have to check
+           for rc==0 *and* the magic number. */
+        if (0 == rc && USNIC_PORT_QUERY_MAGIC == u.qpt.magic) {
+            /* We only support version 1 of the lookup function in
+               this particular version of Open MPI */
+            if (1 == u.qpt.lookup_version) {
+                return USNIC_UDP;
+            } else {
+                return USNIC_UNKNOWN;
+            }
+        } else {
+            return USNIC_L2;
+        }
+    }
+
+#if HAVE_DECL_IBV_TRANSPORT_USNIC_UDP
+    /* If we got the transport type of USNIC_UDP, then it's definitely
+       the UDP transport. */
+    if (IBV_TRANSPORT_USNIC_UDP == device->transport_type) {
+        return USNIC_UDP;
+    }
+#endif
+
+#if HAVE_DECL_IBV_TRANSPORT_USNIC
+    /* If we got the transport type of USNIC, then it's definitely
+       the L2 transport. */
+    if (IBV_TRANSPORT_USNIC == device->transport_type) {
+        return USNIC_L2;
+    }
+#endif
+
+    /* Should never get here -- usnic devices should *always* return
+       one of the 3 above values */
+    return USNIC_UNKNOWN;
+}
+
 /***********************************************************************/
 
 static void check_sanity(char ***if_sanity_list, const char *dev_name, int port)
@@ -239,7 +378,7 @@ opal_list_t *ompi_common_verbs_find_ports(const char *if_include,
     uint32_t i, j;
     opal_list_t *port_list = NULL;
     opal_list_item_t *item;
-    bool want;
+    bool want, dev_is_usnic;
 
     /* Allocate a list to fill */
     port_list = OBJ_NEW(opal_list_t);
@@ -285,9 +424,16 @@ opal_list_t *ompi_common_verbs_find_ports(const char *if_include,
         device = devices[i];
         check_sanity(&if_sanity_list, ibv_get_device_name(device), -1);
 
-        device_context = ibv_open_device(device);
         opal_output_verbose(5, stream, "examining verbs interface: %s",
                             ibv_get_device_name(device));
+
+        dev_is_usnic = false;
+        if ((flags & OMPI_COMMON_VERBS_FLAGS_TRANSPORT_USNIC) ||
+            (flags & OMPI_COMMON_VERBS_FLAGS_TRANSPORT_USNIC_UDP)) {
+            dev_is_usnic = device_is_usnic(device);
+        }
+
+        device_context = ibv_open_device(device);
         if (NULL == device_context) {
             opal_show_help("help-ompi-common-verbs.txt",
                            "ibv_open_device fail", true,
@@ -313,45 +459,68 @@ opal_list_t *ompi_common_verbs_find_ports(const char *if_include,
             check_sanity(&if_sanity_list, ibv_get_device_name(device), j);
         }
 
-        /* Check the the device-specific flags to see if we want this
+        /* Check the device-specific flags to see if we want this
            device */
-        want = true;
+        want = false;
+
+        if (flags & OMPI_COMMON_VERBS_FLAGS_TRANSPORT_USNIC &&
+            dev_is_usnic &&
+            USNIC_L2 == usnic_transport(device, device_context)) {
+            want = true;
+            opal_output_verbose(5, stream,
+                                "verbs interface %s has the right transport (usNIC/L2)",
+                                ibv_get_device_name(device));
+        }
+        if (flags & OMPI_COMMON_VERBS_FLAGS_TRANSPORT_USNIC_UDP &&
+            dev_is_usnic &&
+            USNIC_UDP == usnic_transport(device, device_context)) {
+            want = true;
+            opal_output_verbose(5, stream,
+                                "verbs interface %s has the right transport (usNIC/UDP)",
+                                ibv_get_device_name(device));
+        }
         if (flags & OMPI_COMMON_VERBS_FLAGS_TRANSPORT_IB &&
-            IBV_TRANSPORT_IB != device->transport_type) {
-            opal_output_verbose(5, stream, "verbs interface %s has wrong type (has %s, want IB)",
-                                ibv_get_device_name(device),
-                                transport_name_to_str(device->transport_type));
-            want = false;
+            IBV_TRANSPORT_IB == device->transport_type) {
+            opal_output_verbose(5, stream, "verbs interface %s has right type (IB)",
+                                ibv_get_device_name(device));
+            want = true;
         }
         if (flags & OMPI_COMMON_VERBS_FLAGS_TRANSPORT_IWARP &&
-            IBV_TRANSPORT_IWARP != device->transport_type) {
-            opal_output_verbose(5, stream, "verbs interface %s has wrong type (has %s, want IWARP)",
-                                ibv_get_device_name(device),
-                                transport_name_to_str(device->transport_type));
-            want = false;
+            IBV_TRANSPORT_IWARP == device->transport_type) {
+            opal_output_verbose(5, stream, "verbs interface %s has right type (IWARP)",
+                                ibv_get_device_name(device));
+            want = true;
         }
 
         /* Check for RC or UD QP support */
-        if (flags & OMPI_COMMON_VERBS_FLAGS_RC ||
-            flags & OMPI_COMMON_VERBS_FLAGS_UD) {
+        if (flags & OMPI_COMMON_VERBS_FLAGS_RC) {
             rc = ompi_common_verbs_qp_test(device_context, flags);
-            if (OMPI_ERR_TYPE_MISMATCH == rc) {
-                want = false;
-                opal_output_verbose(5, stream, 
-                                    "verbs interface %s:%d: made an RC QP! we don't want RC-capable devices",
-                                    ibv_get_device_name(device), j);
-            } else if (OMPI_SUCCESS != rc) {
-                want = false;
-                opal_output_verbose(5, stream, 
-                                    "verbs interface %s:%d: failed to make %s QP",
-                                    ibv_get_device_name(device), j,
-                                    ((flags & (OMPI_COMMON_VERBS_FLAGS_RC |
-                                               OMPI_COMMON_VERBS_FLAGS_UD)) ==
-                                     (OMPI_COMMON_VERBS_FLAGS_RC |
-                                      OMPI_COMMON_VERBS_FLAGS_UD)) ? 
-                                    "both UD and RC" :
-                                    (flags & OMPI_COMMON_VERBS_FLAGS_RC) ? 
-                                    "RC" : "UD");
+            if (OMPI_SUCCESS == rc) {
+                want = true;
+                opal_output_verbose(5, stream,
+                                    "verbs interface %s supports RC QPs",
+                                    ibv_get_device_name(device));
+            } else {
+                opal_output_verbose(5, stream,
+                                    "verbs interface %s failed to make RC QP",
+                                    ibv_get_device_name(device));
+            }
+        }
+        if (flags & OMPI_COMMON_VERBS_FLAGS_UD) {
+            rc = ompi_common_verbs_qp_test(device_context, flags);
+            if (OMPI_SUCCESS == rc) {
+                want = true;
+                opal_output_verbose(5, stream,
+                                    "verbs interface %s supports UD QPs",
+                                    ibv_get_device_name(device));
+            } else if (OMPI_ERR_TYPE_MISMATCH == rc) {
+                opal_output_verbose(5, stream,
+                                    "verbs interface %s made an RC QP! we don't want RC-capable devices",
+                                    ibv_get_device_name(device));
+            } else {
+                opal_output_verbose(5, stream,
+                                    "verbs interface %s failed to make UD QP",
+                                    ibv_get_device_name(device));
             }
         }
 
