@@ -27,8 +27,13 @@
 
 int
 ompi_osc_base_select(ompi_win_t *win,
-                    ompi_info_t *info,
-                    ompi_communicator_t *comm)
+                     void **base,
+                     size_t size,
+                     int disp_unit,
+                     ompi_communicator_t *comm,
+                     ompi_info_t *info,
+                     int flavor,
+                     int *model)
 {
     opal_list_item_t *item;
     ompi_osc_base_component_t *best_component = NULL;
@@ -45,7 +50,7 @@ ompi_osc_base_select(ompi_win_t *win,
         ompi_osc_base_component_t *component = (ompi_osc_base_component_t*)
             ((mca_base_component_list_item_t*) item)->cli_component;
 
-        priority = component->osc_query(win, info, comm);
+        priority = component->osc_query(win, base, size, disp_unit, comm, info, flavor);
         if (priority < 0) continue;
         if (priority > best_priority) {
             best_component = component;
@@ -55,5 +60,5 @@ ompi_osc_base_select(ompi_win_t *win,
 
     if (NULL == best_component) return OMPI_ERR_NOT_SUPPORTED;
 
-    return best_component->osc_select(win, info, comm);
+    return best_component->osc_select(win, base, size, disp_unit, comm, info, flavor, model);
 }
