@@ -36,10 +36,10 @@
 #include "opal/util/arch.h"
 #include "ompi/memchecker.h"
 
-#if OMPI_CUDA_SUPPORT
+#if OPAL_CUDA_SUPPORT
 int mca_pml_bfo_cuda_need_buffers(mca_pml_bfo_recv_request_t* recvreq,
                                   mca_btl_base_module_t* btl);
-#endif /* OMPI_CUDA_SUPPORT */
+#endif /* OPAL_CUDA_SUPPORT */
 #if PML_BFO
 #include "pml_bfo_failover.h"
 #endif /* PML_BFO */
@@ -540,15 +540,15 @@ void mca_pml_bfo_recv_request_progress_rget( mca_pml_bfo_recv_request_t* recvreq
      * sender side is already registered. We need to be smarter here, perhaps
      * do couple of RDMA reads */
     if(opal_convertor_need_buffers(&recvreq->req_recv.req_base.req_convertor) == true) {
-#if OMPI_CUDA_SUPPORT
+#if OPAL_CUDA_SUPPORT
         if (mca_pml_bfo_cuda_need_buffers(recvreq, btl)) {
             mca_pml_bfo_recv_request_ack(recvreq, &hdr->hdr_rndv, 0);
             return;
         }
-#else /* OMPI_CUDA_SUPPORT */
+#else /* OPAL_CUDA_SUPPORT */
         mca_pml_bfo_recv_request_ack(recvreq, &hdr->hdr_rndv, 0);
         return;
-#endif /* OMPI_CUDA_SUPPORT */
+#endif /* OPAL_CUDA_SUPPORT */
     }
     
     MCA_PML_BFO_RDMA_FRAG_ALLOC(frag);
@@ -583,7 +583,7 @@ void mca_pml_bfo_recv_request_progress_rget( mca_pml_bfo_recv_request_t* recvreq
     frag->rdma_btl = btl;
 #endif /* PML_BFO */
     frag->rdma_bml = mca_bml_base_btl_array_find(&bml_endpoint->btl_rdma, btl);
-#if OMPI_CUDA_SUPPORT
+#if OPAL_CUDA_SUPPORT
     if( OPAL_UNLIKELY(NULL == frag->rdma_bml) ) {
         if (recvreq->req_recv.req_base.req_convertor.flags & CONVERTOR_CUDA) {
             /* Check to see if this is a CUDA get */
@@ -600,12 +600,12 @@ void mca_pml_bfo_recv_request_progress_rget( mca_pml_bfo_recv_request_t* recvreq
             return;
         }
     }
-#else /* OMPI_CUDA_SUPPORT */
+#else /* OPAL_CUDA_SUPPORT */
     if( OPAL_UNLIKELY(NULL == frag->rdma_bml) ) {
         opal_output(0, "[%s:%d] invalid bml for rdma get", __FILE__, __LINE__);
         ompi_rte_abort(-1, NULL);
     }
-#endif /* OMPI_CUDA_SUPPORT */
+#endif /* OPAL_CUDA_SUPPORT */
     frag->rdma_hdr.hdr_rget = *hdr;
     frag->rdma_req = recvreq;
     frag->rdma_ep = bml_endpoint;
