@@ -319,12 +319,12 @@ mca_pml_bfo_send_request_schedule(mca_pml_bfo_send_request_t* sendreq)
     mca_pml_bfo_send_request_schedule_exclusive(sendreq);
 }
 
-#if OMPI_CUDA_SUPPORT
+#if OPAL_CUDA_SUPPORT
 int mca_pml_bfo_send_request_start_cuda(
     mca_pml_bfo_send_request_t* sendreq, 
     mca_bml_base_btl_t* bml_btl,
     size_t size);
-#endif /* OMPI_CUDA_SUPPORT */
+#endif /* OPAL_CUDA_SUPPORT */
 
 /**
  *  Start the specified request
@@ -410,11 +410,11 @@ mca_pml_bfo_send_request_start_btl( mca_pml_bfo_send_request_t* sendreq,
                                                          MCA_PML_BFO_HDR_FLAGS_CONTIG);
             }
         } else {
-#if OMPI_CUDA_SUPPORT
+#if OPAL_CUDA_SUPPORT
             if (sendreq->req_send.req_base.req_convertor.flags & CONVERTOR_CUDA) {
                 return mca_pml_bfo_send_request_start_cuda(sendreq, bml_btl, size);
             }
-#endif /* OMPI_CUDA_SUPPORT */
+#endif /* OPAL_CUDA_SUPPORT */
             rc = mca_pml_bfo_send_request_start_rndv(sendreq, bml_btl, size, 0);
         }
     }
