@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2012-2013 Los Alamos National Security, LLC.
  *                         All rights reserved.
- * Copyright (c) 2013      Intel, Inc. All rights reserved
+ * Copyright (c) 2013-2014 Intel, Inc. All rights reserved
  *
  * $COPYRIGHT$
  * 
@@ -23,6 +23,8 @@
 #include "ompi/info/info.h"
 struct ompi_proc_t;
 
+#include "opal/threads/threads.h"
+
 #include "orte/types.h"
 #include "orte/mca/errmgr/errmgr.h"
 #include "orte/mca/grpcomm/grpcomm.h"
@@ -62,7 +64,7 @@ typedef orte_ns_cmp_bitmask_t ompi_rte_cmp_bitmask_t;
 /* Collective objects and operations */
 #define ompi_rte_collective_t orte_grpcomm_collective_t
 typedef orte_grpcomm_coll_id_t ompi_rte_collective_id_t;
-#define ompi_rte_modex(a) orte_grpcomm.modex(a)
+OMPI_DECLSPEC int ompi_rte_modex(ompi_rte_collective_t *coll);
 #define ompi_rte_barrier(a) orte_grpcomm.barrier(a)
 
 /* Process info struct and values */
@@ -122,6 +124,22 @@ typedef orte_rml_tag_t ompi_rml_tag_t;
 #define OMPI_RML_PERSISTENT      ORTE_RML_PERSISTENT
 #define OMPI_RML_NON_PERSISTENT  ORTE_RML_NON_PERSISTENT
 
+typedef struct {
+    ompi_rte_component_t super;
+    bool direct_modex;
+    opal_mutex_t lock;
+    opal_list_t modx_reqs;
+} ompi_rte_orte_component_t;
+
+typedef struct {
+    opal_list_item_t super;
+    opal_mutex_t lock;
+    opal_condition_t cond;
+    bool active;
+    orte_process_name_t peer;
+} ompi_orte_tracker_t;
+OBJ_CLASS_DECLARATION(ompi_orte_tracker_t);
+
 END_C_DECLS
 
 #endif /* MCA_OMPI_RTE_ORTE_H */
