@@ -108,6 +108,7 @@
 #define MPI_Comm_split PMPI_Comm_split
 #define MPI_Comm_split_type PMPI_Comm_split_type
 #define MPI_Comm_test_inter PMPI_Comm_test_inter
+#define MPI_Compare_and_swap PMPI_Compare_and_swap
 #define MPI_Dims_create PMPI_Dims_create
 #define MPI_Errhandler_c2f PMPI_Errhandler_c2f
 #define MPI_Errhandler_f2c PMPI_Errhandler_f2c
@@ -118,6 +119,7 @@
 #define MPI_Error_class PMPI_Error_class
 #define MPI_Error_string PMPI_Error_string
 #define MPI_Exscan PMPI_Exscan
+#define MPI_Fetch_and_op PMPI_Fetch_and_op
 #define MPI_Iexscan PMPI_Iexscan
 #define MPI_File_c2f PMPI_File_c2f
 #define MPI_File_call_errhandler PMPI_File_call_errhandler
@@ -188,6 +190,7 @@
 #define MPI_Get_elements PMPI_Get_elements
 #define MPI_Get_elements_x PMPI_Get_elements_x
 #define MPI_Get PMPI_Get
+#define MPI_Get_accumulate PMPI_Get_accumulate
 #define MPI_Get_library_version PMPI_Get_library_version
 #define MPI_Get_processor_name PMPI_Get_processor_name
 #define MPI_Get_version PMPI_Get_version
@@ -272,6 +275,7 @@
 #define MPI_Publish_name PMPI_Publish_name 
 #define MPI_Put PMPI_Put
 #define MPI_Query_thread PMPI_Query_thread
+#define MPI_Raccumulate PMPI_Raccumulate
 #define MPI_Recv_init PMPI_Recv_init 
 #define MPI_Recv PMPI_Recv 
 #define MPI_Reduce PMPI_Reduce
@@ -286,6 +290,9 @@
 #define MPI_Request_f2c PMPI_Request_f2c
 #define MPI_Request_free PMPI_Request_free
 #define MPI_Request_get_status PMPI_Request_get_status 
+#define MPI_Rget PMPI_Rget
+#define MPI_Rget_accumulate PMPI_Rget_accumulate
+#define MPI_Rput PMPI_Rput
 #define MPI_Rsend_init PMPI_Rsend_init 
 #define MPI_Rsend PMPI_Rsend 
 #define MPI_Scan PMPI_Scan
@@ -361,29 +368,44 @@
 #define MPI_Waitall PMPI_Waitall
 #define MPI_Waitany PMPI_Waitany
 #define MPI_Waitsome PMPI_Waitsome
+#define MPI_Win_allocate PMPI_Win_allocate
+#define MPI_Win_allocate_shared PMPI_Win_allocate_shared
+#define MPI_Win_attach PMPI_Win_attach
 #define MPI_Win_c2f PMPI_Win_c2f 
 #define MPI_Win_call_errhandler PMPI_Win_call_errhandler 
 #define MPI_Win_complete PMPI_Win_complete
 #define MPI_Win_create_errhandler PMPI_Win_create_errhandler 
 #define MPI_Win_create_keyval PMPI_Win_create_keyval
 #define MPI_Win_create PMPI_Win_create
+#define MPI_Win_create_dynamic PMPI_Win_create_dynamic
 #define MPI_Win_delete_attr PMPI_Win_delete_attr 
+#define MPI_Win_detach PMPI_Win_detach
 #define MPI_Win_f2c PMPI_Win_f2c
 #define MPI_Win_fence PMPI_Win_fence
+#define MPI_Win_flush PMPI_Win_flush
+#define MPI_Win_flush_all PMPI_Win_flush_all
+#define MPI_Win_flush_local PMPI_Win_flush_local
+#define MPI_Win_flush_local_all PMPI_Win_flush_local_all
 #define MPI_Win_free_keyval PMPI_Win_free_keyval 
 #define MPI_Win_free PMPI_Win_free 
 #define MPI_Win_get_attr PMPI_Win_get_attr 
 #define MPI_Win_get_errhandler PMPI_Win_get_errhandler
 #define MPI_Win_get_group PMPI_Win_get_group
+#define MPI_Win_get_info PMPI_Win_get_info
 #define MPI_Win_get_name PMPI_Win_get_name
 #define MPI_Win_lock PMPI_Win_lock
+#define MPI_Win_lock_all PMPI_Win_lock_all
 #define MPI_Win_post PMPI_Win_post 
 #define MPI_Win_set_attr PMPI_Win_set_attr
 #define MPI_Win_set_errhandler PMPI_Win_set_errhandler 
+#define MPI_Win_set_info PMPI_Win_set_info 
 #define MPI_Win_set_name PMPI_Win_set_name 
+#define MPI_Win_shared_query PMPI_Win_shared_query
 #define MPI_Win_start PMPI_Win_start
+#define MPI_Win_sync PMPI_Win_sync
 #define MPI_Win_test PMPI_Win_test
 #define MPI_Win_unlock PMPI_Win_unlock
+#define MPI_Win_unlock_all PMPI_Win_unlock_all
 #define MPI_Win_wait PMPI_Win_wait
 #define MPI_Wtick PMPI_Wtick 
 #define MPI_Wtime PMPI_Wtime
