@@ -34,7 +34,7 @@ JNIEXPORT jlong JNICALL Java_mpi_File_close(
     return (jlong)file;
 }
 
-JNIEXPORT void JNICALL Java_mpi_File_delete_1jni(
+JNIEXPORT void JNICALL Java_mpi_File_delete(
         JNIEnv *env, jclass clazz, jstring jfilename, jlong info)
 {
     const char* filename = (*env)->GetStringUTFChars(env, jfilename, NULL);
@@ -116,82 +116,70 @@ JNIEXPORT void JNICALL Java_mpi_File_setView(
 
 JNIEXPORT void JNICALL Java_mpi_File_readAt(
         JNIEnv *env, jobject jthis, jlong fh, jlong fileOffset,
-        jobject buf, jint offset, jint count, jobject jType, jobject stat)
+        jobject buf, jboolean db, jint offset, jint count,
+        jlong type, jint bType, jlongArray stat)
 {
-    MPI_Datatype type = (MPI_Datatype)((*env)->GetLongField(
-                        env, jType, ompi_java.DatatypeHandle));
-
-    int bType = (*env)->GetIntField(env, jType, ompi_java.DatatypeBaseType);
     void *ptr, *base;
-    ptr = ompi_java_getBufPtr(&base, env, buf, bType, offset);
+    ptr = ompi_java_getBufPtr(&base, env, buf, db, bType, offset);
     MPI_Status status;
 
     int rc = MPI_File_read_at((MPI_File)fh, (MPI_Offset)fileOffset,
                               ptr, count, (MPI_Datatype)type, &status);
 
     ompi_java_exceptionCheck(env, rc);
-    ompi_java_releaseBufPtr(env, buf, base, bType);
-    ompi_java_status_set(&status, env, stat);
+    ompi_java_releaseBufPtr(env, buf, db, base, bType);
+    ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_readAtAll(
         JNIEnv *env, jobject jthis, jlong fh, jlong fileOffset,
-        jobject buf, jint offset, jint count, jobject jType, jobject stat)
+        jobject buf, jboolean db, jint offset, jint count,
+        jlong type, jint bType, jlongArray stat)
 {
-    MPI_Datatype type = (MPI_Datatype)((*env)->GetLongField(
-                        env, jType, ompi_java.DatatypeHandle));
-
-    int bType = (*env)->GetIntField(env, jType, ompi_java.DatatypeBaseType);
     void *ptr, *base;
-    ptr = ompi_java_getBufPtr(&base, env, buf, bType, offset);
+    ptr = ompi_java_getBufPtr(&base, env, buf, db, bType, offset);
     MPI_Status status;
 
     int rc = MPI_File_read_at_all((MPI_File)fh, (MPI_Offset)fileOffset,
                                   ptr, count, (MPI_Datatype)type, &status);
 
     ompi_java_exceptionCheck(env, rc);
-    ompi_java_releaseBufPtr(env, buf, base, bType);
-    ompi_java_status_set(&status, env, stat);
+    ompi_java_releaseBufPtr(env, buf, db, base, bType);
+    ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_writeAt(
         JNIEnv *env, jobject jthis, jlong fh, jlong fileOffset,
-        jobject buf, jint offset, jint count, jobject jType, jobject stat)
+        jobject buf, jboolean db, jint offset, jint count,
+        jlong type, jint bType, jlongArray stat)
 {
-    MPI_Datatype type = (MPI_Datatype)((*env)->GetLongField(
-                        env, jType, ompi_java.DatatypeHandle));
-
-    int bType = (*env)->GetIntField(env, jType, ompi_java.DatatypeBaseType);
     void *ptr, *base;
-    ptr = ompi_java_getBufPtr(&base, env, buf, bType, offset);
+    ptr = ompi_java_getBufPtr(&base, env, buf, db, bType, offset);
     MPI_Status status;
 
     int rc = MPI_File_write_at((MPI_File)fh, (MPI_Offset)fileOffset,
                                ptr, count, (MPI_Datatype)type, &status);
 
     ompi_java_exceptionCheck(env, rc);
-    ompi_java_releaseReadBufPtr(env, buf, base, bType);
-    ompi_java_status_set(&status, env, stat);
+    ompi_java_releaseReadBufPtr(env, buf, db, base, bType);
+    ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_writeAtAll(
         JNIEnv *env, jobject jthis, jlong fh, jlong fileOffset,
-        jobject buf, jint offset, jint count, jobject jType, jobject stat)
+        jobject buf, jboolean db, jint offset, jint count,
+        jlong type, jint bType, jlongArray stat)
 {
-    MPI_Datatype type = (MPI_Datatype)((*env)->GetLongField(
-                        env, jType, ompi_java.DatatypeHandle));
-
-    int bType = (*env)->GetIntField(env, jType, ompi_java.DatatypeBaseType);
     void *ptr, *base;
-    ptr = ompi_java_getBufPtr(&base, env, buf, bType, offset);
+    ptr = ompi_java_getBufPtr(&base, env, buf, db, bType, offset);
     MPI_Status status;
 
     int rc = MPI_File_write_at_all((MPI_File)fh, (MPI_Offset)fileOffset,
                                    ptr, count, (MPI_Datatype)type, &status);
 
     ompi_java_exceptionCheck(env, rc);
-    ompi_java_releaseReadBufPtr(env, buf, base, bType);
-    ompi_java_status_set(&status, env, stat);
+    ompi_java_releaseReadBufPtr(env, buf, db, base, bType);
+    ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT jlong JNICALL Java_mpi_File_iReadAt(
@@ -223,83 +211,67 @@ JNIEXPORT jlong JNICALL Java_mpi_File_iWriteAt(
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_read(
-        JNIEnv *env, jobject jthis, jlong fh, jobject buf,
-        jint offset, jint count, jobject jType, jobject stat)
+        JNIEnv *env, jobject jthis, jlong fh, jobject buf, jboolean db,
+        jint offset, jint count, jlong type, jint bType, jlongArray stat)
 {
-    MPI_Datatype type = (MPI_Datatype)((*env)->GetLongField(
-                        env, jType, ompi_java.DatatypeHandle));
-
-    int bType = (*env)->GetIntField(env, jType, ompi_java.DatatypeBaseType);
     void *ptr, *base;
-    ptr = ompi_java_getBufPtr(&base, env, buf, bType, offset);
+    ptr = ompi_java_getBufPtr(&base, env, buf, db, bType, offset);
     MPI_Status status;
 
     int rc = MPI_File_read((MPI_File)fh, ptr, count,
                            (MPI_Datatype)type, &status);
 
     ompi_java_exceptionCheck(env, rc);
-    ompi_java_releaseBufPtr(env, buf, base, bType);
-    ompi_java_status_set(&status, env, stat);
+    ompi_java_releaseBufPtr(env, buf, db, base, bType);
+    ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_readAll(
-        JNIEnv *env, jobject jthis, jlong fh, jobject buf,
-        jint offset, jint count, jobject jType, jobject stat)
+        JNIEnv *env, jobject jthis, jlong fh, jobject buf, jboolean db,
+        jint offset, jint count, jlong type, jint bType, jlongArray stat)
 {
-    MPI_Datatype type = (MPI_Datatype)((*env)->GetLongField(
-                        env, jType, ompi_java.DatatypeHandle));
-
-    int bType = (*env)->GetIntField(env, jType, ompi_java.DatatypeBaseType);
     void *ptr, *base;
-    ptr = ompi_java_getBufPtr(&base, env, buf, bType, offset);
+    ptr = ompi_java_getBufPtr(&base, env, buf, db, bType, offset);
     MPI_Status status;
 
     int rc = MPI_File_read_all((MPI_File)fh, ptr, count,
                                (MPI_Datatype)type, &status);
 
     ompi_java_exceptionCheck(env, rc);
-    ompi_java_releaseBufPtr(env, buf, base, bType);
-    ompi_java_status_set(&status, env, stat);
+    ompi_java_releaseBufPtr(env, buf, db, base, bType);
+    ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_write(
-        JNIEnv *env, jobject jthis, jlong fh, jobject buf,
-        jint offset, jint count, jobject jType, jobject stat)
+        JNIEnv *env, jobject jthis, jlong fh, jobject buf, jboolean db,
+        jint offset, jint count, jlong type, jint bType, jlongArray stat)
 {
-    MPI_Datatype type = (MPI_Datatype)((*env)->GetLongField(
-                        env, jType, ompi_java.DatatypeHandle));
-
-    int bType = (*env)->GetIntField(env, jType, ompi_java.DatatypeBaseType);
     void *ptr, *base;
-    ptr = ompi_java_getBufPtr(&base, env, buf, bType, offset);
+    ptr = ompi_java_getBufPtr(&base, env, buf, db, bType, offset);
     MPI_Status status;
 
     int rc = MPI_File_write((MPI_File)fh, ptr, count,
                             (MPI_Datatype)type, &status);
 
     ompi_java_exceptionCheck(env, rc);
-    ompi_java_releaseReadBufPtr(env, buf, base, bType);
-    ompi_java_status_set(&status, env, stat);
+    ompi_java_releaseReadBufPtr(env, buf, db, base, bType);
+    ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_writeAll(
-        JNIEnv *env, jobject jthis, jlong fh, jobject buf,
-        jint offset, jint count, jobject jType, jobject stat)
+        JNIEnv *env, jobject jthis, jlong fh, jobject buf, jboolean db,
+        jint offset, jint count, jlong type, jint bType, jlongArray stat)
 {
-    MPI_Datatype type = (MPI_Datatype)((*env)->GetLongField(
-                        env, jType, ompi_java.DatatypeHandle));
-
-    int bType = (*env)->GetIntField(env, jType, ompi_java.DatatypeBaseType);
     void *ptr, *base;
-    ptr = ompi_java_getBufPtr(&base, env, buf, bType, offset);
+    ptr = ompi_java_getBufPtr(&base, env, buf, db, bType, offset);
     MPI_Status status;
 
     int rc = MPI_File_write_all((MPI_File)fh, ptr, count,
                                 (MPI_Datatype)type, &status);
 
     ompi_java_exceptionCheck(env, rc);
-    ompi_java_releaseReadBufPtr(env, buf, base, bType);
-    ompi_java_status_set(&status, env, stat);
+    ompi_java_releaseReadBufPtr(env, buf, db, base, bType);
+    ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT jlong JNICALL Java_mpi_File_iRead(
@@ -356,43 +328,35 @@ JNIEXPORT jlong JNICALL Java_mpi_File_getByteOffset(
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_readShared(
-        JNIEnv *env, jobject jthis, jlong fh, jobject buf,
-        jint offset, jint count, jobject jType, jobject stat)
+        JNIEnv *env, jobject jthis, jlong fh, jobject buf, jboolean db,
+        jint offset, jint count, jlong type, jint bType, jlongArray stat)
 {
-    MPI_Datatype type = (MPI_Datatype)((*env)->GetLongField(
-                        env, jType, ompi_java.DatatypeHandle));
-
-    int bType = (*env)->GetIntField(env, jType, ompi_java.DatatypeBaseType);
     void *ptr, *base;
-    ptr = ompi_java_getBufPtr(&base, env, buf, bType, offset);
+    ptr = ompi_java_getBufPtr(&base, env, buf, db, bType, offset);
     MPI_Status status;
 
     int rc = MPI_File_read_shared((MPI_File)fh, ptr, count,
                                   (MPI_Datatype)type, &status);
 
     ompi_java_exceptionCheck(env, rc);
-    ompi_java_releaseBufPtr(env, buf, base, bType);
-    ompi_java_status_set(&status, env, stat);
+    ompi_java_releaseBufPtr(env, buf, db, base, bType);
+    ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_writeShared(
-        JNIEnv *env, jobject jthis, jlong fh, jobject buf,
-        jint offset, jint count, jobject jType, jobject stat)
+        JNIEnv *env, jobject jthis, jlong fh, jobject buf, jboolean db,
+        jint offset, jint count, jlong type, jint bType, jlongArray stat)
 {
-    MPI_Datatype type = (MPI_Datatype)((*env)->GetLongField(
-                        env, jType, ompi_java.DatatypeHandle));
-
-    int bType = (*env)->GetIntField(env, jType, ompi_java.DatatypeBaseType);
     void *ptr, *base;
-    ptr = ompi_java_getBufPtr(&base, env, buf, bType, offset);
+    ptr = ompi_java_getBufPtr(&base, env, buf, db, bType, offset);
     MPI_Status status;
 
     int rc = MPI_File_write_shared((MPI_File)fh, ptr, count,
                                    (MPI_Datatype)type, &status);
 
     ompi_java_exceptionCheck(env, rc);
-    ompi_java_releaseReadBufPtr(env, buf, base, bType);
-    ompi_java_status_set(&status, env, stat);
+    ompi_java_releaseReadBufPtr(env, buf, db, base, bType);
+    ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT jlong JNICALL Java_mpi_File_iReadShared(
@@ -424,43 +388,35 @@ JNIEXPORT jlong JNICALL Java_mpi_File_iWriteShared(
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_readOrdered(
-        JNIEnv *env, jobject jthis, jlong fh, jobject buf,
-        jint offset, jint count, jobject jType, jobject stat)
+        JNIEnv *env, jobject jthis, jlong fh, jobject buf, jboolean db,
+        jint offset, jint count, jlong type, jint bType, jlongArray stat)
 {
-    MPI_Datatype type = (MPI_Datatype)((*env)->GetLongField(
-                        env, jType, ompi_java.DatatypeHandle));
-
-    int bType = (*env)->GetIntField(env, jType, ompi_java.DatatypeBaseType);
     void *ptr, *base;
-    ptr = ompi_java_getBufPtr(&base, env, buf, bType, offset);
+    ptr = ompi_java_getBufPtr(&base, env, buf, db, bType, offset);
     MPI_Status status;
 
     int rc = MPI_File_read_ordered((MPI_File)fh, ptr, count,
                                    (MPI_Datatype)type, &status);
 
     ompi_java_exceptionCheck(env, rc);
-    ompi_java_releaseBufPtr(env, buf, base, bType);
-    ompi_java_status_set(&status, env, stat);
+    ompi_java_releaseBufPtr(env, buf, db, base, bType);
+    ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_writeOrdered(
-        JNIEnv *env, jobject jthis, jlong fh, jobject buf,
-        jint offset, jint count, jobject jType, jobject stat)
+        JNIEnv *env, jobject jthis, jlong fh, jobject buf, jboolean db,
+        jint offset, jint count, jlong type, jint bType, jlongArray stat)
 {
-    MPI_Datatype type = (MPI_Datatype)((*env)->GetLongField(
-                        env, jType, ompi_java.DatatypeHandle));
-
-    int bType = (*env)->GetIntField(env, jType, ompi_java.DatatypeBaseType);
     void *ptr, *base;
-    ptr = ompi_java_getBufPtr(&base, env, buf, bType, offset);
+    ptr = ompi_java_getBufPtr(&base, env, buf, db, bType, offset);
     MPI_Status status;
 
     int rc = MPI_File_write_ordered((MPI_File)fh, ptr, count,
                                     (MPI_Datatype)type, &status);
 
     ompi_java_exceptionCheck(env, rc);
-    ompi_java_releaseReadBufPtr(env, buf, base, bType);
-    ompi_java_status_set(&status, env, stat);
+    ompi_java_releaseReadBufPtr(env, buf, db, base, bType);
+    ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_seekShared(
@@ -481,11 +437,8 @@ JNIEXPORT jlong JNICALL Java_mpi_File_getPositionShared(
 
 JNIEXPORT void JNICALL Java_mpi_File_readAtAllBegin(
         JNIEnv *env, jobject jthis, jlong fh, jlong offset,
-        jobject buf, jint count, jobject jType)
+        jobject buf, jint count, jlong type)
 {
-    MPI_Datatype type = (MPI_Datatype)((*env)->GetLongField(
-                        env, jType, ompi_java.DatatypeHandle));
-
     void *ptr = (*env)->GetDirectBufferAddress(env, buf);
 
     int rc = MPI_File_read_at_all_begin((MPI_File)fh, (MPI_Offset)offset,
@@ -494,22 +447,19 @@ JNIEXPORT void JNICALL Java_mpi_File_readAtAllBegin(
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_readAtAllEnd(
-        JNIEnv *env, jobject jthis, jlong fh, jobject buf, jobject stat)
+        JNIEnv *env, jobject jthis, jlong fh, jobject buf, jlongArray stat)
 {
     MPI_Status status;
     void *ptr = (*env)->GetDirectBufferAddress(env, buf);
     int rc = MPI_File_read_at_all_end((MPI_File)fh, ptr, &status);
     ompi_java_exceptionCheck(env, rc);
-    ompi_java_status_set(&status, env, stat);
+    ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_writeAtAllBegin(
         JNIEnv *env, jobject jthis, jlong fh, jlong fileOffset,
-        jobject buf, jint count, jobject jType)
+        jobject buf, jint count, jlong type)
 {
-    MPI_Datatype type = (MPI_Datatype)((*env)->GetLongField(
-                        env, jType, ompi_java.DatatypeHandle));
-
     void *ptr = (*env)->GetDirectBufferAddress(env, buf);
 
     int rc = MPI_File_write_at_all_begin((MPI_File)fh, (MPI_Offset)fileOffset,
@@ -518,22 +468,19 @@ JNIEXPORT void JNICALL Java_mpi_File_writeAtAllBegin(
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_writeAtAllEnd(
-        JNIEnv *env, jobject jthis, jlong fh, jobject buf, jobject stat)
+        JNIEnv *env, jobject jthis, jlong fh, jobject buf, jlongArray stat)
 {
     MPI_Status status;
     void *ptr = (*env)->GetDirectBufferAddress(env, buf);
     int rc = MPI_File_write_at_all_end((MPI_File)fh, ptr, &status);
     ompi_java_exceptionCheck(env, rc);
-    ompi_java_status_set(&status, env, stat);
+    ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_readAllBegin(
         JNIEnv *env, jobject jthis, jlong fh,
-        jobject buf, jint count, jobject jType)
+        jobject buf, jint count, jlong type)
 {
-    MPI_Datatype type = (MPI_Datatype)((*env)->GetLongField(
-                        env, jType, ompi_java.DatatypeHandle));
-
     void *ptr = (*env)->GetDirectBufferAddress(env, buf);
 
     int rc = MPI_File_read_all_begin(
@@ -543,22 +490,19 @@ JNIEXPORT void JNICALL Java_mpi_File_readAllBegin(
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_readAllEnd(
-        JNIEnv *env, jobject jthis, jlong fh, jobject buf, jobject stat)
+        JNIEnv *env, jobject jthis, jlong fh, jobject buf, jlongArray stat)
 {
     MPI_Status status;
     void *ptr = (*env)->GetDirectBufferAddress(env, buf);
     int rc = MPI_File_read_all_end((MPI_File)fh, ptr, &status);
     ompi_java_exceptionCheck(env, rc);
-    ompi_java_status_set(&status, env, stat);
+    ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_writeAllBegin(
         JNIEnv *env, jobject jthis, jlong fh,
-        jobject buf, jint count, jobject jType)
+        jobject buf, jint count, jlong type)
 {
-    MPI_Datatype type = (MPI_Datatype)((*env)->GetLongField(
-                        env, jType, ompi_java.DatatypeHandle));
-
     void *ptr = (*env)->GetDirectBufferAddress(env, buf);
 
     int rc = MPI_File_write_all_begin(
@@ -568,22 +512,19 @@ JNIEXPORT void JNICALL Java_mpi_File_writeAllBegin(
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_writeAllEnd(
-        JNIEnv *env, jobject jthis, jlong fh, jobject buf, jobject stat)
+        JNIEnv *env, jobject jthis, jlong fh, jobject buf, jlongArray stat)
 {
     MPI_Status status;
     void *ptr = (*env)->GetDirectBufferAddress(env, buf);
     int rc = MPI_File_write_all_end((MPI_File)fh, ptr, &status);
     ompi_java_exceptionCheck(env, rc);
-    ompi_java_status_set(&status, env, stat);
+    ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_readOrderedBegin(
         JNIEnv *env, jobject jthis, jlong fh,
-        jobject buf, jint count, jobject jType)
+        jobject buf, jint count, jlong type)
 {
-    MPI_Datatype type = (MPI_Datatype)((*env)->GetLongField(
-                        env, jType, ompi_java.DatatypeHandle));
-
     void *ptr = (*env)->GetDirectBufferAddress(env, buf);
 
     int rc = MPI_File_read_ordered_begin(
@@ -593,22 +534,19 @@ JNIEXPORT void JNICALL Java_mpi_File_readOrderedBegin(
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_readOrderedEnd(
-        JNIEnv *env, jobject jthis, jlong fh, jobject buf, jobject stat)
+        JNIEnv *env, jobject jthis, jlong fh, jobject buf, jlongArray stat)
 {
     MPI_Status status;
     void *ptr = (*env)->GetDirectBufferAddress(env, buf);
     int rc = MPI_File_read_ordered_end((MPI_File)fh, ptr, &status);
     ompi_java_exceptionCheck(env, rc);
-    ompi_java_status_set(&status, env, stat);
+    ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_writeOrderedBegin(
         JNIEnv *env, jobject jthis, jlong fh,
-        jobject buf, jint count, jobject jType)
+        jobject buf, jint count, jlong type)
 {
-    MPI_Datatype type = (MPI_Datatype)((*env)->GetLongField(
-                        env, jType, ompi_java.DatatypeHandle));
-
     void *ptr = (*env)->GetDirectBufferAddress(env, buf);
 
     int rc = MPI_File_write_ordered_begin(
@@ -618,13 +556,13 @@ JNIEXPORT void JNICALL Java_mpi_File_writeOrderedBegin(
 }
 
 JNIEXPORT void JNICALL Java_mpi_File_writeOrderedEnd(
-        JNIEnv *env, jobject jthis, jlong fh, jobject buf, jobject stat)
+        JNIEnv *env, jobject jthis, jlong fh, jobject buf, jlongArray stat)
 {
     MPI_Status status;
     void *ptr = (*env)->GetDirectBufferAddress(env, buf);
     int rc = MPI_File_write_ordered_end((MPI_File)fh, ptr, &status);
     ompi_java_exceptionCheck(env, rc);
-    ompi_java_status_set(&status, env, stat);
+    ompi_java_status_set(env, stat, &status);
 }
 
 JNIEXPORT jint JNICALL Java_mpi_File_getTypeExtent(
