@@ -1,3 +1,11 @@
+/*
+ * IMPLEMENTATION DETAILS
+ * 
+ * All methods with buffers that can be direct or non direct have
+ * a companion argument 'db' which is true if the buffer is direct.
+ * 
+ * Checking if a buffer is direct is faster in Java than C.
+ */
 package mpi;
 
 import java.nio.*;
@@ -10,6 +18,12 @@ import static mpi.MPI.assertDirectBuffer;
  */
 public final class File
 {
+// Auxiliary status data.
+// It's used to avoid creating status objects in the C side.
+// It also avoids setting objects attributes in the C side.
+// Calling java methods and setting object attributes from C is very slow.
+private long[] status = Status.newData();
+
 private long handle;
 private FileView view = new FileView(0, MPI.BYTE, MPI.BYTE, "native");
 private Status beginStatus;
@@ -65,7 +79,7 @@ private native long close(long fh) throws MPIException;
 public static void delete(String filename) throws MPIException
 {
     MPI.check();
-    delete_jni(filename, Info.NULL);
+    delete(filename, Info.NULL);
 }
 
 /**
@@ -77,10 +91,10 @@ public static void delete(String filename) throws MPIException
 public static void delete(String filename, Info info) throws MPIException
 {
     MPI.check();
-    delete_jni(filename, info.handle);
+    delete(filename, info.handle);
 }
 
-private static native void delete_jni(String filename, long info)
+private static native void delete(String filename, long info)
         throws MPIException;
 
 /**
@@ -236,21 +250,23 @@ public Status readAt(long offset, Object buf, int count, Datatype type)
 {
     MPI.check();
     int off = 0;
+    boolean db = false;
 
-    if(isHeapBuffer(buf))
+    if(buf instanceof Buffer && !(db = ((Buffer)buf).isDirect()))
     {
         off = ((Buffer)buf).arrayOffset();
         buf = ((Buffer)buf).array();
     }
 
-    Status stat = new Status();
-    readAt(handle, offset, buf, off, count, type, stat);
-    return stat;
+    readAt(handle, offset, buf, db, off, count,
+           type.handle, type.baseType, status);
+
+    return newStatus();
 }
 
 private native void readAt(
-        long fh, long fileOffset, Object buf, int offset,
-        int count, Datatype type, Status stat) throws MPIException;
+        long fh, long fileOffset, Object buf, boolean db, int offset,
+        int count, long type, int baseType, long[] stat) throws MPIException;
 
 /**
  * Java binding of {@code MPI_FILE_READ_AT_ALL}.
@@ -266,21 +282,23 @@ public Status readAtAll(long offset, Object buf, int count, Datatype type)
 {
     MPI.check();
     int off = 0;
+    boolean db = false;
 
-    if(isHeapBuffer(buf))
+    if(buf instanceof Buffer && !(db = ((Buffer)buf).isDirect()))
     {
         off = ((Buffer)buf).arrayOffset();
         buf = ((Buffer)buf).array();
     }
 
-    Status stat = new Status();
-    readAtAll(handle, offset, buf, off, count, type, stat);
-    return stat;
+    readAtAll(handle, offset, buf, db, off, count,
+              type.handle, type.baseType, status);
+
+    return newStatus();
 }
 
 private native void readAtAll(
-        long fh, long fileOffset, Object buf, int offset,
-        int count, Datatype type, Status stat) throws MPIException;
+        long fh, long fileOffset, Object buf, boolean db, int offset,
+        int count, long type, int baseType, long[] stat) throws MPIException;
 
 /**
  * Java binding of {@code MPI_FILE_WRITE_AT}.
@@ -296,21 +314,23 @@ public Status writeAt(long offset, Object buf, int count, Datatype type)
 {
     MPI.check();
     int off = 0;
+    boolean db = false;
 
-    if(isHeapBuffer(buf))
+    if(buf instanceof Buffer && !(db = ((Buffer)buf).isDirect()))
     {
         off = ((Buffer)buf).arrayOffset();
         buf = ((Buffer)buf).array();
     }
 
-    Status stat = new Status();
-    writeAt(handle, offset, buf, off, count, type, stat);
-    return stat;
+    writeAt(handle, offset, buf, db, off, count,
+            type.handle, type.baseType, status);
+
+    return newStatus();
 }
 
 private native void writeAt(
-        long fh, long fileOffset, Object buf, int offset,
-        int count, Datatype type, Status stat) throws MPIException;
+        long fh, long fileOffset, Object buf, boolean db, int offset,
+        int count, long type, int baseType, long[] stat) throws MPIException;
 
 /**
  * Java binding of {@code MPI_FILE_WRITE_AT_ALL}.
@@ -326,21 +346,23 @@ public Status writeAtAll(long offset, Object buf, int count, Datatype type)
 {
     MPI.check();
     int off = 0;
+    boolean db = false;
 
-    if(isHeapBuffer(buf))
+    if(buf instanceof Buffer && !(db = ((Buffer)buf).isDirect()))
     {
         off = ((Buffer)buf).arrayOffset();
         buf = ((Buffer)buf).array();
     }
 
-    Status stat = new Status();
-    writeAtAll(handle, offset, buf, off, count, type, stat);
-    return stat;
+    writeAtAll(handle, offset, buf, db, off, count,
+               type.handle, type.baseType, status);
+
+    return newStatus();
 }
 
 private native void writeAtAll(
-        long fh, long fileOffset, Object buf, int offset,
-        int count, Datatype type, Status stat) throws MPIException;
+        long fh, long fileOffset, Object buf, boolean db, int offset,
+        int count, long type, int baseType, long[] stat) throws MPIException;
 
 /**
  * Java binding of {@code MPI_FILE_IREAD_AT}.
@@ -396,21 +418,21 @@ public Status read(Object buf, int count, Datatype type) throws MPIException
 {
     MPI.check();
     int off = 0;
+    boolean db = false;
 
-    if(isHeapBuffer(buf))
+    if(buf instanceof Buffer && !(db = ((Buffer)buf).isDirect()))
     {
         off = ((Buffer)buf).arrayOffset();
         buf = ((Buffer)buf).array();
     }
 
-    Status stat = new Status();
-    read(handle, buf, off, count, type, stat);
-    return stat;
+    read(handle, buf, db, off, count, type.handle, type.baseType, status);
+    return newStatus();
 }
 
 private native void read(
-        long fh, Object buf, int offset,
-        int count, Datatype type, Status stat) throws MPIException;
+        long fh, Object buf, boolean db, int offset,
+        int count, long type, int baseType, long[] stat) throws MPIException;
 
 /**
  * Java binding of {@code MPI_FILE_READ_ALL}.
@@ -424,21 +446,21 @@ public Status readAll(Object buf, int count, Datatype type) throws MPIException
 {
     MPI.check();
     int off = 0;
+    boolean db = false;
 
-    if(isHeapBuffer(buf))
+    if(buf instanceof Buffer && !(db = ((Buffer)buf).isDirect()))
     {
         off = ((Buffer)buf).arrayOffset();
         buf = ((Buffer)buf).array();
     }
 
-    Status stat = new Status();
-    readAll(handle, buf, off, count, type, stat);
-    return stat;
+    readAll(handle, buf, db, off, count, type.handle, type.baseType, status);
+    return newStatus();
 }
 
 private native void readAll(
-        long fh, Object buf, int offset,
-        int count, Datatype type, Status stat) throws MPIException;
+        long fh, Object buf, boolean db, int offset,
+        int count, long type, int baseType, long[] stat) throws MPIException;
 
 /**
  * Java binding of {@code MPI_FILE_WRITE}.
@@ -452,21 +474,21 @@ public Status write(Object buf, int count, Datatype type) throws MPIException
 {
     MPI.check();
     int off = 0;
+    boolean db = false;
 
-    if(isHeapBuffer(buf))
+    if(buf instanceof Buffer && !(db = ((Buffer)buf).isDirect()))
     {
         off = ((Buffer)buf).arrayOffset();
         buf = ((Buffer)buf).array();
     }
 
-    Status stat = new Status();
-    write(handle, buf, off, count, type, stat);
-    return stat;
+    write(handle, buf, db, off, count, type.handle, type.baseType, status);
+    return newStatus();
 }
 
 private native void write(
-        long fh, Object buf, int offset,
-        int count, Datatype type, Status stat) throws MPIException;
+        long fh, Object buf, boolean db, int offset,
+        int count, long type, int baseType, long[] stat) throws MPIException;
 
 /**
  * Java binding of {@code MPI_FILE_WRITE_ALL}.
@@ -480,21 +502,21 @@ public Status writeAll(Object buf, int count, Datatype type) throws MPIException
 {
     MPI.check();
     int off = 0;
+    boolean db = false;
 
-    if(isHeapBuffer(buf))
+    if(buf instanceof Buffer && !(db = ((Buffer)buf).isDirect()))
     {
         off = ((Buffer)buf).arrayOffset();
         buf = ((Buffer)buf).array();
     }
 
-    Status stat = new Status();
-    writeAll(handle, buf, off, count, type, stat);
-    return stat;
+    writeAll(handle, buf,db,off, count, type.handle, type.baseType, status);
+    return newStatus();
 }
 
 private native void writeAll(
-        long fh, Object buf, int offset,
-        int count, Datatype type, Status stat) throws MPIException;
+        long fh, Object buf, boolean db, int offset,
+        int count, long type, int baseType, long[] stat) throws MPIException;
 
 /**
  * Java binding of {@code MPI_FILE_IREAD}.
@@ -586,21 +608,21 @@ public Status readShared(Object buf, int count, Datatype type)
 {
     MPI.check();
     int off = 0;
+    boolean db = false;
 
-    if(isHeapBuffer(buf))
+    if(buf instanceof Buffer && !(db = ((Buffer)buf).isDirect()))
     {
         off = ((Buffer)buf).arrayOffset();
         buf = ((Buffer)buf).array();
     }
 
-    Status stat = new Status();
-    readShared(handle, buf, off, count, type, stat);
-    return stat;
+    readShared(handle, buf, db, off, count, type.handle, type.baseType, status);
+    return newStatus();
 }
 
 private native void readShared(
-        long fh, Object buf, int offset, int count, Datatype type, Status stat)
-        throws MPIException;
+        long fh, Object buf, boolean db, int offset, int count,
+        long type, int baseType, long[] stat) throws MPIException;
 
 /**
  * Java binding of {@code MPI_FILE_WRITE_SHARED}.
@@ -615,21 +637,21 @@ public Status writeShared(Object buf, int count, Datatype type)
 {
     MPI.check();
     int off = 0;
+    boolean db = false;
 
-    if(isHeapBuffer(buf))
+    if(buf instanceof Buffer && !(db = ((Buffer)buf).isDirect()))
     {
         off = ((Buffer)buf).arrayOffset();
         buf = ((Buffer)buf).array();
     }
 
-    Status stat = new Status();
-    writeShared(handle, buf, off, count, type, stat);
-    return stat;
+    writeShared(handle, buf,db,off, count, type.handle, type.baseType, status);
+    return newStatus();
 }
 
 private native void writeShared(
-        long fh, Object buf, int offset, int count, Datatype type, Status stat)
-        throws MPIException;
+        long fh, Object buf, boolean db, int offset, int count,
+        long type, int baseType, long[] stat) throws MPIException;
 
 /**
  * Java binding of {@code MPI_FILE_IREAD_SHARED}.
@@ -682,21 +704,21 @@ public Status readOrdered(Object buf, int count, Datatype type)
 {
     MPI.check();
     int off = 0;
+    boolean db = false;
 
-    if(isHeapBuffer(buf))
+    if(buf instanceof Buffer && !(db = ((Buffer)buf).isDirect()))
     {
         off = ((Buffer)buf).arrayOffset();
         buf = ((Buffer)buf).array();
     }
 
-    Status stat = new Status();
-    readOrdered(handle, buf, off, count, type, stat);
-    return stat;
+    readOrdered(handle, buf,db,off, count, type.handle, type.baseType, status);
+    return newStatus();
 }
 
 private native void readOrdered(
-        long fh, Object buf, int offset, int count, Datatype type, Status stat)
-        throws MPIException;
+        long fh, Object buf, boolean db, int offset, int count,
+        long type, int baseType, long[] stat) throws MPIException;
 
 /**
  * Java binding of {@code MPI_FILE_WRITE_ORDERED}.
@@ -711,21 +733,21 @@ public Status writeOrdered(Object buf, int count, Datatype type)
 {
     MPI.check();
     int off = 0;
+    boolean db = false;
 
-    if(isHeapBuffer(buf))
+    if(buf instanceof Buffer && !(db = ((Buffer)buf).isDirect()))
     {
         off = ((Buffer)buf).arrayOffset();
         buf = ((Buffer)buf).array();
     }
 
-    Status stat = new Status();
-    writeOrdered(handle, buf, off, count, type, stat);
-    return stat;
+    writeOrdered(handle, buf,db,off, count, type.handle, type.baseType, status);
+    return newStatus();
 }
 
 private native void writeOrdered(
-        long fh, Object buf, int offset, int count, Datatype type, Status stat)
-        throws MPIException;
+        long fh, Object buf, boolean db, int offset, int count,
+        long type, int baseType, long[] stat) throws MPIException;
 
 /**
  * Java binding of {@code MPI_FILE_SEEK_SHARED}.
@@ -770,7 +792,7 @@ public void readAtAllBegin(long offset, Object buf, int count, Datatype type)
 
     if(isDirectBuffer(buf))
     {
-        readAtAllBegin(handle, offset, buf, count, type);
+        readAtAllBegin(handle, offset, buf, count, type.handle);
     }
     else
     {
@@ -782,14 +804,15 @@ public void readAtAllBegin(long offset, Object buf, int count, Datatype type)
             buf = ((Buffer)buf).array();
         }
 
-        Status stat = new Status();
-        readAtAll(handle, offset, buf, off, count, type, stat);
-        beginStatus = stat;
+        readAtAll(handle, offset, buf, false, off, count,
+                  type.handle, type.baseType, status);
+
+        beginStatus = newStatus();
     }
 }
 
 private native void readAtAllBegin(
-        long fh, long offset, Object buf, int count, Datatype type)
+        long fh, long offset, Object buf, int count, long type)
         throws MPIException;
 
 /**
@@ -804,9 +827,8 @@ public Status readAtAllEnd(Object buf) throws MPIException
 
     if(isDirectBuffer(buf))
     {
-        Status stat = new Status();
-        readAtAllEnd(handle, buf, stat);
-        return stat;
+        readAtAllEnd(handle, buf, status);
+        return newStatus();
     }
     else
     {
@@ -814,7 +836,7 @@ public Status readAtAllEnd(Object buf) throws MPIException
     }
 }
 
-private native void readAtAllEnd(long fh, Object buf, Status stat)
+private native void readAtAllEnd(long fh, Object buf, long[] stat)
         throws MPIException;
 
 /**
@@ -832,7 +854,7 @@ public void writeAtAllBegin(long offset, Object buf, int count, Datatype type)
 
     if(isDirectBuffer(buf))
     {
-        writeAtAllBegin(handle, offset, buf, count, type);
+        writeAtAllBegin(handle, offset, buf, count, type.handle);
     }
     else
     {
@@ -844,14 +866,15 @@ public void writeAtAllBegin(long offset, Object buf, int count, Datatype type)
             buf = ((Buffer)buf).array();
         }
 
-        Status stat = new Status();
-        writeAtAll(handle, offset, buf, off, count, type, stat);
-        beginStatus = stat;
+        writeAtAll(handle, offset, buf, false, off, count,
+                   type.handle, type.baseType, status);
+
+        beginStatus = newStatus();
     }
 }
 
 private native void writeAtAllBegin(
-        long fh, long fileOffset, Object buf, int count, Datatype type)
+        long fh, long fileOffset, Object buf, int count, long type)
         throws MPIException;
 
 /**
@@ -866,9 +889,8 @@ public Status writeAtAllEnd(Object buf) throws MPIException
 
     if(isDirectBuffer(buf))
     {
-        Status stat = new Status();
-        writeAtAllEnd(handle, buf, stat);
-        return stat;
+        writeAtAllEnd(handle, buf, status);
+        return newStatus();
     }
     else
     {
@@ -876,7 +898,7 @@ public Status writeAtAllEnd(Object buf) throws MPIException
     }
 }
 
-private native void writeAtAllEnd(long fh, Object buf, Status stat)
+private native void writeAtAllEnd(long fh, Object buf, long[] stat)
         throws MPIException;
 
 /**
@@ -893,7 +915,7 @@ public void readAllBegin(Object buf, int count, Datatype type)
 
     if(isDirectBuffer(buf))
     {
-        readAllBegin(handle, buf, count, type);
+        readAllBegin(handle, buf, count, type.handle);
     }
     else
     {
@@ -905,13 +927,14 @@ public void readAllBegin(Object buf, int count, Datatype type)
             buf = ((Buffer)buf).array();
         }
 
-        Status stat = new Status();
-        readAll(handle, buf, off, count, type, stat);
-        beginStatus = stat;
+        readAll(handle, buf, false, off, count,
+                type.handle, type.baseType, status);
+
+        beginStatus = newStatus();
     }
 }
 
-private native void readAllBegin(long fh, Object buf, int count, Datatype type)
+private native void readAllBegin(long fh, Object buf, int count, long type)
         throws MPIException;
 
 /**
@@ -926,9 +949,8 @@ public Status readAllEnd(Object buf) throws MPIException
 
     if(isDirectBuffer(buf))
     {
-        Status stat = new Status();
-        readAllEnd(handle, buf, stat);
-        return stat;
+        readAllEnd(handle, buf, status);
+        return newStatus();
     }
     else
     {
@@ -936,7 +958,7 @@ public Status readAllEnd(Object buf) throws MPIException
     }
 }
 
-private native void readAllEnd(long fh, Object buf, Status stat)
+private native void readAllEnd(long fh, Object buf, long[] stat)
         throws MPIException;
 
 /**
@@ -953,7 +975,7 @@ public void writeAllBegin(Object buf, int count, Datatype type)
 
     if(isDirectBuffer(buf))
     {
-        writeAllBegin(handle, buf, count, type);
+        writeAllBegin(handle, buf, count, type.handle);
     }
     else
     {
@@ -965,13 +987,14 @@ public void writeAllBegin(Object buf, int count, Datatype type)
             buf = ((Buffer)buf).array();
         }
 
-        Status stat = new Status();
-        writeAll(handle, buf, off, count, type, stat);
-        beginStatus = stat;
+        writeAll(handle, buf, false, off, count,
+                 type.handle, type.baseType, status);
+
+        beginStatus = newStatus();
     }
 }
 
-private native void writeAllBegin(long fh, Object buf, int count, Datatype type)
+private native void writeAllBegin(long fh, Object buf, int count, long type)
         throws MPIException;
 
 /**
@@ -986,9 +1009,8 @@ public Status writeAllEnd(Object buf) throws MPIException
 
     if(isDirectBuffer(buf))
     {
-        Status stat = new Status();
-        writeAllEnd(handle, buf, stat);
-        return stat;
+        writeAllEnd(handle, buf, status);
+        return newStatus();
     }
     else
     {
@@ -996,7 +1018,7 @@ public Status writeAllEnd(Object buf) throws MPIException
     }
 }
 
-private native void writeAllEnd(long fh, Object buf, Status stat)
+private native void writeAllEnd(long fh, Object buf, long[] stat)
         throws MPIException;
 
 /**
@@ -1013,7 +1035,7 @@ public void readOrderedBegin(Object buf, int count, Datatype type)
 
     if(isDirectBuffer(buf))
     {
-        readOrderedBegin(handle, buf, count, type);
+        readOrderedBegin(handle, buf, count, type.handle);
     }
     else
     {
@@ -1025,14 +1047,14 @@ public void readOrderedBegin(Object buf, int count, Datatype type)
             buf = ((Buffer)buf).array();
         }
 
-        Status stat = new Status();
-        readOrdered(handle, buf, off, count, type, stat);
-        beginStatus = stat;
+        readOrdered(handle, buf, false, off, count,
+                    type.handle, type.baseType, status);
+
+        beginStatus = newStatus();
     }
 }
 
-private native void readOrderedBegin(
-        long fh, Object buf, int count, Datatype type)
+private native void readOrderedBegin(long fh, Object buf, int count, long type)
         throws MPIException;
 
 /**
@@ -1047,9 +1069,8 @@ public Status readOrderedEnd(Object buf) throws MPIException
 
     if(isDirectBuffer(buf))
     {
-        Status stat = new Status();
-        readOrderedEnd(handle, buf, stat);
-        return stat;
+        readOrderedEnd(handle, buf, status);
+        return newStatus();
     }
     else
     {
@@ -1057,7 +1078,7 @@ public Status readOrderedEnd(Object buf) throws MPIException
     }
 }
 
-private native void readOrderedEnd(long fh, Object buf, Status stat)
+private native void readOrderedEnd(long fh, Object buf, long[] stat)
         throws MPIException;
 
 /**
@@ -1074,7 +1095,7 @@ public void writeOrderedBegin(Object buf, int count, Datatype type)
 
     if(isDirectBuffer(buf))
     {
-        writeOrderedBegin(handle, buf, count, type);
+        writeOrderedBegin(handle, buf, count, type.handle);
     }
     else
     {
@@ -1086,14 +1107,14 @@ public void writeOrderedBegin(Object buf, int count, Datatype type)
             buf = ((Buffer)buf).array();
         }
 
-        Status stat = new Status();
-        writeOrdered(handle, buf, off, count, type, stat);
-        beginStatus = stat;
+        writeOrdered(handle, buf, false, off, count,
+                     type.handle, type.baseType, status);
+
+        beginStatus = newStatus();
     }
 }
 
-private native void writeOrderedBegin(
-        long fh, Object buf, int count, Datatype type)
+private native void writeOrderedBegin(long fh, Object buf, int count, long type)
         throws MPIException;
 
 /**
@@ -1108,9 +1129,8 @@ public Status writeOrderedEnd(Object buf) throws MPIException
 
     if(isDirectBuffer(buf))
     {
-        Status stat = new Status();
-        writeOrderedEnd(handle, buf, stat);
-        return stat;
+        writeOrderedEnd(handle, buf, status);
+        return newStatus();
     }
     else
     {
@@ -1118,7 +1138,7 @@ public Status writeOrderedEnd(Object buf) throws MPIException
     }
 }
 
-private native void writeOrderedEnd(long fh, Object buf, Status stat)
+private native void writeOrderedEnd(long fh, Object buf, long[] stat)
         throws MPIException;
 
 private Status getBeginStatus()
@@ -1140,8 +1160,7 @@ public int getTypeExtent(Datatype type) throws MPIException
     return getTypeExtent(handle, type.handle) / type.baseSize;
 }
 
-private native int getTypeExtent(long fh, long type)
-        throws MPIException;
+private native int getTypeExtent(long fh, long type) throws MPIException;
 
 /**
  * Java binding of {@code MPI_FILE_SET_ATOMICITY}.
@@ -1169,4 +1188,11 @@ public void sync() throws MPIException
 
 private native void sync(long handle) throws MPIException;
 
+private Status newStatus()
+{
+    Status s = new Status(status);
+    status = Status.newData();
+    return s;
+}
+
 } // File
