@@ -1,6 +1,6 @@
 /*
- * Copyright (c) 2011-2013 Cisco Systems, Inc.  All rights reserved.
- * Copyright (c) 2013      Intel, Inc. All rights reserved.
+ * Copyright (c) 2011-2014 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2013-2014 Intel, Inc. All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -192,6 +192,9 @@ static int opal_hwloc_base_open(mca_base_open_flag_t flags)
         }
 
         if (opal_hwloc_base_bind_to_core) {
+            opal_show_help("help-opal-hwloc-base.txt", "deprecated", true,
+                           "--bind-to-core", "--bind-to core",
+                           "hwloc_base_bind_to_core", "hwloc_base_binding_policy=core");
             /* set binding policy to core - error if something else already set */
             if (OPAL_BINDING_POLICY_IS_SET(opal_hwloc_binding_policy) &&
                 OPAL_GET_BINDING_POLICY(opal_hwloc_binding_policy) != OPAL_BIND_TO_CORE) {
@@ -204,6 +207,9 @@ static int opal_hwloc_base_open(mca_base_open_flag_t flags)
         }
 
         if (opal_hwloc_base_bind_to_socket) {
+            opal_show_help("help-opal-hwloc-base.txt", "deprecated", true,
+                           "--bind-to-socket", "--bind-to socket",
+                           "hwloc_base_bind_to_socket", "hwloc_base_binding_policy=socket");
             /* set binding policy to socket - error if something else already set */
             if (OPAL_BINDING_POLICY_IS_SET(opal_hwloc_binding_policy) &&
                 OPAL_GET_BINDING_POLICY(opal_hwloc_binding_policy) != OPAL_BIND_TO_SOCKET) {
@@ -237,6 +243,11 @@ static int opal_hwloc_base_open(mca_base_open_flag_t flags)
             }
         }
 
+        /* if we are binding to hwthreads, then we must use hwthreads as cpus */
+        if (OPAL_GET_BINDING_POLICY(opal_hwloc_binding_policy) == OPAL_BIND_TO_HWTHREAD) {
+            opal_hwloc_use_hwthreads_as_cpus = true;
+        }
+
         /* to support tools such as ompi_info, add the components
          * to a list
          */
