@@ -75,11 +75,11 @@ opal_installdirs_base_open(mca_base_open_flag_t flags)
         CONDITIONAL_COPY(opal_install_dirs, component->install_dirs_data, 
                          mandir);
         CONDITIONAL_COPY(opal_install_dirs, component->install_dirs_data,
-                         pkgdatadir);
+                         ompidatadir);
         CONDITIONAL_COPY(opal_install_dirs, component->install_dirs_data, 
-                         pkglibdir);
+                         ompilibdir);
         CONDITIONAL_COPY(opal_install_dirs, component->install_dirs_data, 
-                         pkgincludedir);
+                         ompiincludedir);
     }
 
     /* expand out all the fields */
@@ -111,12 +111,12 @@ opal_installdirs_base_open(mca_base_open_flag_t flags)
         opal_install_dirs_expand_setup(opal_install_dirs.infodir);
     opal_install_dirs.mandir = 
         opal_install_dirs_expand_setup(opal_install_dirs.mandir);
-    opal_install_dirs.pkgdatadir = 
-        opal_install_dirs_expand_setup(opal_install_dirs.pkgdatadir);
-    opal_install_dirs.pkglibdir = 
-        opal_install_dirs_expand_setup(opal_install_dirs.pkglibdir);
-    opal_install_dirs.pkgincludedir = 
-        opal_install_dirs_expand_setup(opal_install_dirs.pkgincludedir);
+    opal_install_dirs.ompidatadir = 
+        opal_install_dirs_expand_setup(opal_install_dirs.ompidatadir);
+    opal_install_dirs.ompilibdir = 
+        opal_install_dirs_expand_setup(opal_install_dirs.ompilibdir);
+    opal_install_dirs.ompiincludedir = 
+        opal_install_dirs_expand_setup(opal_install_dirs.ompiincludedir);
 
 #if 0
     fprintf(stderr, "prefix:         %s\n", opal_install_dirs.prefix);
@@ -162,9 +162,9 @@ opal_installdirs_base_close(void)
     free(opal_install_dirs.includedir);
     free(opal_install_dirs.infodir);
     free(opal_install_dirs.mandir);
-    free(opal_install_dirs.pkgdatadir);
-    free(opal_install_dirs.pkglibdir);
-    free(opal_install_dirs.pkgincludedir);
+    free(opal_install_dirs.ompidatadir);
+    free(opal_install_dirs.ompilibdir);
+    free(opal_install_dirs.ompiincludedir);
 
     return mca_base_framework_components_close (&opal_installdirs_base_framework, NULL);
 }
