@@ -50,6 +50,7 @@
 #endif /* HAVE_SYS_STAT_H */
 
 #include "opal/constants.h"
+#include "opal/util/alfg.h"
 #include "opal/util/output.h"
 #include "opal/util/path.h"
 #include "opal/util/show_help.h"
@@ -211,7 +212,8 @@ get_uniq_file_name(const char *base_path, const char *hash_key)
     char *uniq_name_buf = NULL;
     unsigned long str_hash = 0;
     pid_t my_pid;
-    int rand_num;
+    opal_rng_buff_t rand_buff;
+    uint32_t rand_num;
 
     /* invalid argument */
     if (NULL == hash_key) {
@@ -223,8 +225,8 @@ get_uniq_file_name(const char *base_path, const char *hash_key)
     }
 
     my_pid = getpid();
-    srand((unsigned int)(time(NULL) + my_pid));
-    rand_num = rand() % 1024;
+    opal_srand(&rand_buff,((uint32_t)(time(NULL) + my_pid)));
+    rand_num = opal_rand(&rand_buff) % 1024;
     str_hash = sdbm_hash((unsigned char *)hash_key);
     /* build the name */
     snprintf(uniq_name_buf, OPAL_PATH_MAX, "%s/open_mpi_shmem_mmap.%d_%lu_%d",
