@@ -66,6 +66,9 @@
 
 #include "orte/mca/ess/base/base.h"
 
+static void* orte_progress_thread_engine(opal_object_t *obj);
+static bool progress_thread_running = false;
+
 int orte_ess_base_app_setup(bool db_restrict_local)
 {
     int ret;
@@ -91,6 +94,9 @@ int orte_ess_base_app_setup(bool db_restrict_local)
         }
     }
 
+    /* get a separate orte event base */
+    orte_event_base = opal_event_base_create();
+ 
     /* open and setup the state machine */
     if (ORTE_SUCCESS != (ret = mca_base_framework_open(&orte_state_base_framework, 0))) {
         ORTE_ERROR_LOG(ret);
@@ -193,6 +199,17 @@ int orte_ess_base_app_setup(bool db_restrict_local)
         goto error;
     }
     
+    /* construct the thread object */
+    OBJ_CONSTRUCT(&orte_progress_thread, opal_thread_t);
+    /* fork off a thread to progress it */
+    orte_progress_thread.t_run = orte_progress_thread_engine;
+    progress_thread_running = true;
+    if (OPAL_SUCCESS != (ret = opal_thread_start(&orte_progress_thread))) {
+        error = "orte progress thread start";
+        progress_thread_running = false;
+        goto error;
+    }
+
     /* enable communication via the rml */
     if (ORTE_SUCCESS != (ret = orte_rml.enable_comm())) {
         ORTE_ERROR_LOG(ret);
@@ -218,18 +235,13 @@ int orte_ess_base_app_setup(bool db_restrict_local)
         }
         
         /* Once the session directory location has been established, set
-         the opal_output env file location to be in the
-         proc-specific session directory. */
+           the opal_output env file location to be in the
+           proc-specific session directory. */
         opal_output_set_output_file_info(orte_process_info.proc_session_dir,
                                          "output-", NULL, NULL);
     }
     
-    /* setup the routed info - the selected routed component
-     * will know what to do. Some may put us in a blocking
-     * receive here so they can get ALL of the contact info
-     * from our peers. Others may just find the local daemon's
-     * contact info and immediately return.
-     */
+    /* setup the routed info  */
     if (ORTE_SUCCESS != (ret = orte_routed.init_routes(ORTE_PROC_MY_NAME->jobid, NULL))) {
         ORTE_ERROR_LOG(ret);
         error = "orte_routed.init_routes";
@@ -284,7 +296,7 @@ int orte_ess_base_app_setup(bool db_restrict_local)
         error = "orte_dfs_base_open";
         goto error;
     }
-   if (ORTE_SUCCESS != (ret = orte_dfs_base_select())) {
+    if (ORTE_SUCCESS != (ret = orte_dfs_base_select())) {
         ORTE_ERROR_LOG(ret);
         error = "orte_dfs_base_select";
         goto error;
@@ -292,7 +304,13 @@ int orte_ess_base_app_setup(bool db_restrict_local)
 
     return ORTE_SUCCESS;
     
-error:
+ error:
+    if (!progress_thread_running) {
+        /* can't send the help message, so ensure it
+         * comes out locally
+         */
+        orte_show_help_finalize();
+    }
     orte_show_help("help-orte-runtime.txt",
                    "orte_init:startup:internal-failure",
                    true, error, ORTE_ERROR_NAME(ret), ret);
@@ -318,9 +336,28 @@ int orte_ess_base_app_finalize(void)
     (void) mca_base_framework_close(&opal_db_base_framework);
     (void) mca_base_framework_close(&orte_dfs_base_framework);
     (void) mca_base_framework_close(&orte_routed_base_framework);
+
+    if (progress_thread_running) {
+        /* we had to leave the progress thread running until
+         * we closed the routed framework as that closure
+         * sends a "sync" message to the local daemon. it
+         * is now safe to stop the progress thread
+         */
+        orte_event_base_active = false;
+        /* break the event loop */
+        opal_event_base_loopbreak(orte_event_base);
+        /* wait for thread to exit */
+        opal_thread_join(&orte_progress_thread, NULL);
+        OBJ_DESTRUCT(&orte_progress_thread);
+        progress_thread_running = false;
+    }
+
     (void) mca_base_framework_close(&orte_rml_base_framework);
     (void) mca_base_framework_close(&orte_oob_base_framework);
 
+    /* release the event base */
+    opal_event_base_free(orte_event_base);
+
     orte_session_dir_finalize(ORTE_PROC_MY_NAME);
         
     return ORTE_SUCCESS;    
@@ -381,9 +418,13 @@ void orte_ess_base_app_abort(int status, bool report)
     
     /* If we were asked to report this termination, do so - except
      * in cases of abnormal termination ordered by the RTE as
-     * this means we can't rely on being able to communicate
+     * this means we can't rely on being able to communicate. Also,
+     * since singletons don't start an HNP unless necessary, and
+     * direct-launched procs don't have daemons at all, only send
+     * the message if routing is enabled as this indicates we
+     * have someone to send to
      */
-    if (report && !orte_abnormal_term_ordered) {
+    if (report && !orte_abnormal_term_ordered && orte_routing_is_enabled) {
         buf = OBJ_NEW(opal_buffer_t);
         opal_dss.pack(buf, &cmd, 1, ORTE_DAEMON_CMD);
         orte_rml.send_buffer_nb(ORTE_PROC_MY_DAEMON, buf, ORTE_RML_TAG_DAEMON, orte_rml_send_callback, NULL);
@@ -408,3 +449,11 @@ void orte_ess_base_app_abort(int status, bool report)
     /* Now Exit */
     exit(status);
 }
+
+static void* orte_progress_thread_engine(opal_object_t *obj)
+{
+    while (orte_event_base_active) {
+        opal_event_loop(orte_event_base, OPAL_EVLOOP_ONCE);
+    }
+    return OPAL_THREAD_CANCELLED;
+}
