@@ -302,6 +302,11 @@ static int rte_init(void)
     }
     /* Setup the communication infrastructure */
     
+    /* clear the session directory just in case there are
+     * stale directories laying around
+     */
+    orte_session_dir_cleanup(ORTE_JOBID_WILDCARD);
+
     /*
      * OOB Layer
      */
@@ -915,7 +920,7 @@ static void clean_abort(int fd, short flags, void *arg)
        (which is a Bad Thing), so we can't call it directly.
        Instead, we have to exit this handler and setup to call
        job_completed() after this. */
-    ORTE_FORCED_TERMINATE(ORTE_ERROR_DEFAULT_EXIT_CODE);
+    orte_plm.terminate_orteds();;
 }
 
 static struct timeval current, last={0,0};
