@@ -12,7 +12,7 @@
  * Copyright (c) 2008-2012 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2012-2013 Los Alamos National Security, LLC.
  *                         All rights reserved. 
- * Copyright (c) 2013      Intel, Inc.  All rights reserved.
+ * Copyright (c) 2013-2014 Intel, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -95,6 +95,7 @@ static int rte_init(void)
     int *ranks=NULL;
     orte_jobid_t jobid;
     char *rmluri;
+    orte_process_name_t ldr;
 
     /* run the prolog */
     if (ORTE_SUCCESS != (ret = orte_ess_base_std_prolog())) {
@@ -113,6 +114,12 @@ static int rte_init(void)
 #endif
 
     if (ORTE_PROC_IS_DAEMON) {  /* I am a daemon, launched by mpirun */
+        /* ensure that we always exit with a non-zero status
+         * so that Slurm and other such RMs will terminate the
+         * job if any daemon exits, whether normal termination or not
+         */
+        ORTE_UPDATE_EXIT_STATUS(ORTE_ERROR_DEFAULT_EXIT_CODE);
+
         /* we had to be given a jobid */
         if (NULL == orte_ess_base_jobid) {
             error = "missing jobid";
@@ -333,6 +340,14 @@ static int rte_init(void)
                 break;
             }
         }
+        /* store the name of the local leader */
+        ldr.jobid = ORTE_PROC_MY_NAME->jobid;
+        ldr.vpid = ranks[0];
+        if (ORTE_SUCCESS != (ret = opal_db.store((opal_identifier_t*)ORTE_PROC_MY_NAME, OPAL_SCOPE_INTERNAL,
+                                                 OPAL_DB_LOCALLDR, (opal_identifier_t*)&ldr, OPAL_ID_T))) {
+            error = "storing local leader";
+            goto error;
+        }
         free(ranks);
 
         /* setup process binding */
