@@ -200,6 +200,10 @@ static bool odls_default_child_died(orte_proc_t *child)
      * the default 1s actually means 'somwhere between 0 and 1s'. */
     end = time(NULL) + orte_odls_globals.timeout_before_sigkill + 1;
     do {
+        OPAL_OUTPUT_VERBOSE((2, orte_odls_base_framework.framework_output,
+                             "%s odls:default:WAITPID CHECKING PID %d WITH TIMEOUT %d SECONDS",
+                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), (int)(child->pid),
+                             orte_odls_globals.timeout_before_sigkill + 1));
         ret = waitpid(child->pid, &child->exit_code, WNOHANG);
         if (child->pid == ret) {
             OPAL_OUTPUT_VERBOSE((2, orte_odls_base_framework.framework_output,
@@ -222,7 +226,9 @@ static bool odls_default_child_died(orte_proc_t *child)
 	     * which will occasionally trip the timeout for cases that
 	     * are right on the edge.)
              */
-
+            OPAL_OUTPUT_VERBOSE((2, orte_odls_base_framework.framework_output,
+                                 "%s odls:default:WAITPID INDICATES PID %d MAY HAVE ALREADY EXITED",
+                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), (int)(child->pid)));
 	    /* Do nothing, process still alive */
         } else if (-1 == ret && ECHILD == errno) {
             /* The pid no longer exists, so we'll call this "good
@@ -249,9 +255,21 @@ static bool odls_default_child_died(orte_proc_t *child)
 
 static int odls_default_kill_local(pid_t pid, int signum)
 {
-    if (orte_forward_job_control) {
-        pid = -pid;
+    pid_t pgrp;
+
+#if HAVE_SETPGID
+    pgrp = getpgid(pid);
+    if (-1 != pgrp) {
+        /* target the lead process of the process
+         * group so we ensure that the signal is
+         * seen by all members of that group. This
+         * ensures that the signal is seen by any
+         * child processes our child may have
+         * started
+         */
+        pid = pgrp;
     }
+#endif
     if (0 != kill(pid, signum)) {
         if (ESRCH != errno) {
             OPAL_OUTPUT_VERBOSE((2, orte_odls_base_framework.framework_output,
@@ -430,9 +448,32 @@ static int do_child(orte_app_context_t* context,
 #if OPAL_HAVE_HWLOC
         {
             hwloc_cpuset_t cpuset;
+            hwloc_obj_t root;
+            opal_hwloc_topo_data_t *sum;
 
             /* Set process affinity, if given */
-            if (NULL != child->cpu_bitmap) {
+            if (NULL == child->cpu_bitmap) {
+                /* if the daemon is bound, then we need to "free" this proc */
+                if (NULL != orte_daemon_cores) {
+                    root = hwloc_get_root_obj(opal_hwloc_topology);
+                    if (NULL == root->userdata) {
+                        send_warn_show_help(write_fd,
+                                            "help-orte-odls-default.txt", "incorrectly bound",
+                                            orte_process_info.nodename, context->app,
+                                            __FILE__, __LINE__);
+                    }
+                    sum = (opal_hwloc_topo_data_t*)root->userdata;
+                    /* bind this proc to all available processors */
+                    hwloc_set_cpubind(opal_hwloc_topology, sum->available, 0);
+                }
+                if (opal_hwloc_report_bindings) {
+                    opal_output(0, "MCW rank %d is not bound (or bound to all available processors)", child->name.vpid);
+                    /* avoid reporting it twice */
+                    (void) mca_base_var_env_name ("hwloc_base_report_bindings", &param);
+                    opal_unsetenv(param, &environ_copy);
+                    free(param);
+                }
+            } else {
                 if (0 == strlen(child->cpu_bitmap)) {
                     /* this proc is not bound */
                     if (opal_hwloc_report_bindings) {
@@ -442,11 +483,22 @@ static int do_child(orte_app_context_t* context,
                         opal_unsetenv(param, &environ_copy);
                         free(param);
                     }
-                    /* Set an info MCA param that tells
-                       the launched processes that it was bound by us (e.g., so that
-                       MPI_INIT doesn't try to bind itself) */
-                    (void) mca_base_var_env_name ("orte_bound_at_launch", &param);
-                    opal_setenv(param, "1", true, &environ_copy);
+                    /* if the daemon is bound, then we need to "free" this proc */
+                    if (NULL != orte_daemon_cores) {
+                        root = hwloc_get_root_obj(opal_hwloc_topology);
+                        if (NULL == root->userdata) {
+                            send_warn_show_help(write_fd,
+                                                "help-orte-odls-default.txt", "incorrectly bound",
+                                                orte_process_info.nodename, context->app,
+                                                __FILE__, __LINE__);
+                        }
+                        sum = (opal_hwloc_topo_data_t*)root->userdata;
+                        /* bind this proc to all available processors */
+                        hwloc_set_cpubind(opal_hwloc_topology, sum->available, 0);
+                    }
+                    /* provide a nice string representation of what we bound to */
+                    (void) mca_base_var_env_name ("orte_base_applied_binding", &param);
+                    opal_setenv(param, child->cpu_bitmap, true, &environ_copy);
                     free(param);
                     goto PROCEED;
                 }
@@ -460,7 +512,7 @@ static int do_child(orte_app_context_t* context,
                         msg = "failed to convert bitmap list to hwloc bitmap";
                     }
                     if (OPAL_BINDING_REQUIRED(jobdat->map->binding) &&
-                        (OPAL_BIND_GIVEN & jobdat->map->binding)) {
+                        OPAL_BINDING_POLICY_IS_SET(jobdat->map->binding)) {
                         /* If binding is required and a binding directive was explicitly
                          * given (i.e., we are not binding due to a default policy),
                          * send an error up the pipe (which exits -- it doesn't return).
@@ -481,7 +533,7 @@ static int do_child(orte_app_context_t* context,
                 /* bind as specified */
                 rc = hwloc_set_cpubind(opal_hwloc_topology, cpuset, 0);
                 /* if we got an error and this wasn't a default binding policy, then report it */
-                if (rc < 0  && (OPAL_BIND_GIVEN & jobdat->map->binding)) {
+                if (rc < 0  && OPAL_BINDING_POLICY_IS_SET(jobdat->map->binding)) {
                     if (errno == ENOSYS) {
                         msg = "hwloc indicates cpu binding not supported";
                     } else if (errno == EXDEV) {
@@ -516,10 +568,10 @@ static int do_child(orte_app_context_t* context,
                         opal_output(0, "MCW rank %d is not bound",
                                     child->name.vpid);
                     } else {
-                        if (OPAL_ERR_NOT_BOUND == opal_hwloc_base_cset2str(tmp1, sizeof(tmp1), mycpus)) {
+                        if (OPAL_ERR_NOT_BOUND == opal_hwloc_base_cset2str(tmp1, sizeof(tmp1), opal_hwloc_topology, mycpus)) {
                             opal_output(0, "MCW rank %d is not bound (or bound to all available processors)", child->name.vpid);
                         } else {
-                            opal_hwloc_base_cset2mapstr(tmp2, sizeof(tmp2), mycpus);
+                            opal_hwloc_base_cset2mapstr(tmp2, sizeof(tmp2), opal_hwloc_topology, mycpus);
                             opal_output(0, "MCW rank %d bound to %s: %s",
                                         child->name.vpid, tmp1, tmp2);
                         }
@@ -534,7 +586,7 @@ static int do_child(orte_app_context_t* context,
                  * anything unless the user actually specified the binding policy
                  */
                 rc = opal_hwloc_base_set_process_membind_policy();
-                if (ORTE_SUCCESS != rc  && (OPAL_BIND_GIVEN & jobdat->map->binding)) {
+                if (ORTE_SUCCESS != rc  && OPAL_BINDING_POLICY_IS_SET(jobdat->map->binding)) {
                     if (errno == ENOSYS) {
                         msg = "hwloc indicates memory binding not supported";
                     } else if (errno == EXDEV) {
@@ -557,17 +609,6 @@ static int do_child(orte_app_context_t* context,
                         goto PROCEED;
                     }
                 }
-                /* Set an info MCA param that tells
-                   the launched processes that it was bound by us (e.g., so that
-                   MPI_INIT doesn't try to bind itself) */
-                (void) mca_base_var_env_name ("orte_bound_at_launch", &param);
-                opal_setenv(param, "1", true, &environ_copy);
-                free(param);
-                /* ...and provide a nice string representation of what we
-                   bound to */
-                (void) mca_base_var_env_name ("orte_base_applied_binding", &param);
-                opal_setenv(param, child->cpu_bitmap, true, &environ_copy);
-                free (param);
             }
         }
 #endif
@@ -860,6 +901,9 @@ static int odls_default_fork_local_proc(orte_app_context_t* context,
     
     if (pid == 0) {
 	close(p[0]);
+#if HAVE_SETPGID
+        setpgid(0, 0);
+#endif
         do_child(context, child, environ_copy, jobdat, p[1], opts);
         /* Does not return */
     } 
