@@ -11,9 +11,9 @@
  *                         All rights reserved.
  * Copyright (c) 2006-2013 Los Alamos National Security, LLC. 
  *                         All rights reserved.
- * Copyright (c) 2009-2012 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2009-2014 Cisco Systems, Inc.  All rights reserved.
  * Copyright (c) 2011      Oak Ridge National Labs.  All rights reserved.
- * Copyright (c) 2013      Intel, Inc.  All rights reserved.
+ * Copyright (c) 2013-2014 Intel, Inc.  All rights reserved.
  * Copyright (c) 2014      NVIDIA Corporation.  All rights reserved.
  * $COPYRIGHT$
  *
@@ -111,7 +111,7 @@ mca_oob_tcp_component_t mca_oob_tcp_component = {
             MCA_BASE_METADATA_PARAM_CHECKPOINT
         },
         0,   // reserve space for an assigned index
-        100, // default priority of this transport
+        30, // default priority of this transport
         component_available,
         component_startup,
         component_shutdown,
@@ -132,8 +132,8 @@ static int tcp_component_open(void)
     if (ORTE_PROC_IS_HNP) {
         OBJ_CONSTRUCT(&mca_oob_tcp_component.listen_thread, opal_thread_t);
         mca_oob_tcp_component.listen_thread_active = false;
-        mca_oob_tcp_component.listen_thread_tv.tv_sec = 1;
-        mca_oob_tcp_component.listen_thread_tv.tv_usec = 0;
+        mca_oob_tcp_component.listen_thread_tv.tv_sec = 0;
+        mca_oob_tcp_component.listen_thread_tv.tv_usec = 300000;
     }
     mca_oob_tcp_component.addr_count = 0;
     OBJ_CONSTRUCT(&mca_oob_tcp_component.modules, opal_pointer_array_t);
@@ -185,6 +185,7 @@ static int tcp_component_close(void)
     /* cleanup modules */
     for (i=0; i < mca_oob_tcp_component.modules.size; i++) {
         if (NULL != (mod = (mca_oob_tcp_module_t*)opal_pointer_array_get_item(&mca_oob_tcp_component.modules, i))) {
+            free(mod->if_name);
             free(mod);
         }
     }
@@ -560,6 +561,10 @@ static bool component_available(void)
          */
 
         if (add_this_nic) {
+            opal_output_verbose(10, orte_oob_base_framework.framework_output,
+                                "%s oob:tcp:init creating module for %s address on interface %s",
+                                ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                                (AF_INET == my_ss.ss_family) ? "V4" : "V6", name);
             /* we want to support this interface, so create a module for it */
             if (ORTE_SUCCESS != (rc = mca_oob_tcp_create(kindex, name))) {
                 ORTE_ERROR_LOG(rc);
@@ -569,13 +574,27 @@ static bool component_available(void)
 
         /* add this address to our connections */
         if (AF_INET == my_ss.ss_family) {
+            opal_output_verbose(10, orte_oob_base_framework.framework_output,
+                                "%s oob:tcp:init adding %s to our list of %s connections",
+                                ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                                opal_net_get_hostname((struct sockaddr*) &my_ss),
+                                (AF_INET == my_ss.ss_family) ? "V4" : "V6");
             opal_argv_append_nosize(&mca_oob_tcp_component.ipv4conns, opal_net_get_hostname((struct sockaddr*) &my_ss));
         } else if (AF_INET6 == my_ss.ss_family) {
 #if OPAL_ENABLE_IPV6
+            opal_output_verbose(10, orte_oob_base_framework.framework_output,
+                                "%s oob:tcp:init adding %s to our list of %s connections",
+                                ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                                opal_net_get_hostname((struct sockaddr*) &my_ss),
+                                (AF_INET == my_ss.ss_family) ? "V4" : "V6");
             opal_argv_append_nosize(&mca_oob_tcp_component.ipv6conns, opal_net_get_hostname((struct sockaddr*) &my_ss));
 #endif
+        } else {
+            opal_output_verbose(10, orte_oob_base_framework.framework_output,
+                                "%s oob:tcp:init ignoring %s from out list of connections",
+                                ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                                opal_net_get_hostname((struct sockaddr*) &my_ss));
         }
-
     }
 
     /* cleanup */
@@ -588,8 +607,6 @@ static bool component_available(void)
             orte_show_help("help-oob-tcp.txt", "no-included-found", true, mca_oob_tcp_component.if_include);
         } else if (excluding) {
             orte_show_help("help-oob-tcp.txt", "excluded-all", true, mca_oob_tcp_component.if_exclude);
-        } else {
-            orte_show_help("help-oob-tcp.txt", "no-interfaces-avail", true);
         }
         return false;
     }
@@ -663,10 +680,10 @@ static int component_send(orte_rml_send_t *msg)
     opal_output_verbose(5, orte_oob_base_framework.framework_output,
                         "%s oob:tcp:send_nb to peer %s:%d",
                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                        ORTE_NAME_PRINT(&msg->peer), msg->tag);
+                        ORTE_NAME_PRINT(&msg->dst), msg->tag);
 
     /* do we know some way of potentially reaching this peer? */
-    memcpy(&ui64, (char*)&msg->peer, sizeof(uint64_t));
+    memcpy(&ui64, (char*)&msg->dst, sizeof(uint64_t));
     if (OPAL_SUCCESS != opal_hash_table_get_value_uint64(&mca_oob_tcp_component.peers,
                                                          ui64, (void**)&pr)) {
         /* nope - let someone else try */
@@ -782,7 +799,7 @@ static int component_set_addr(orte_process_name_t *peer,
     mca_oob_tcp_component_peer_t *pr;
     uint16_t af_family = AF_UNSPEC;
     uint64_t ui64;
-    bool found;
+    bool found, assigned;
 
     memcpy(&ui64, (char*)peer, sizeof(uint64_t));
     /* cycle across component parts and see if one belongs to us */
@@ -849,24 +866,51 @@ static int component_set_addr(orte_process_name_t *peer,
 
 
         /* cycle across the provided addrs */
+        assigned = false;
         for (j=0; NULL != addrs[j]; j++) {
+            /* if they gave us "localhost", then just take the first conn on our list */
+            if (0 == strcasecmp(addrs[j], "localhost")) {
+#if OPAL_ENABLE_IPV6
+                if (AF_INET6 == af_family) {
+                    if (NULL == mca_oob_tcp_component.ipv6conns ||
+                        NULL == mca_oob_tcp_component.ipv6conns[0]) {
+                        continue;
+                    }
+                    host = mca_oob_tcp_component.ipv6conns[0];
+                } else {
+#endif
+                    if (NULL == mca_oob_tcp_component.ipv4conns ||
+                        NULL == mca_oob_tcp_component.ipv4conns[0]) {
+                        continue;
+                    }
+                    host = mca_oob_tcp_component.ipv4conns[0];
+#if OPAL_ENABLE_IPV6
+                }
+#endif
+            } else {
+                host = addrs[j];
+            }
             /* lookup the kernel index of this address */
-            if (0 >= (k = opal_ifaddrtokindex(addrs[j]))) {
+            if (0 >= (k = opal_ifaddrtokindex(host))) {
                 opal_output_verbose(OOB_TCP_DEBUG_CONNECT, orte_oob_base_framework.framework_output,
                                     "%s UNFOUND KERNEL INDEX %d FOR ADDRESS %s",
-                                    ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), k, addrs[j]);
+                                    ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), k, host);
                 /* we don't have an interface on this subnet - ignore it */
                 continue;
             }
             if (NULL == (mod = (mca_oob_tcp_module_t*)opal_pointer_array_get_item(&mca_oob_tcp_component.modules, k))) {
                 opal_output_verbose(OOB_TCP_DEBUG_CONNECT, orte_oob_base_framework.framework_output,
                                     "%s NO MODULE AT KINDEX %d FOR ADDRESS %s",
-                                    ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), k, addrs[j]);
+                                    ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), k, host);
                 continue;
             }
             /* record that this peer may be reachable via this module, but don't assign
              * the peer to this module until later when we actually connect
              */
+            opal_output_verbose(OOB_TCP_DEBUG_CONNECT, orte_oob_base_framework.framework_output,
+                                "%s PEER %s MAY BE REACHABLE USING MODULE AT KINDEX %d INTERFACE %s",
+                                ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                                ORTE_NAME_PRINT(peer), k, mod->if_name);
             if (OPAL_SUCCESS != opal_hash_table_get_value_uint64(&mca_oob_tcp_component.peers,
                                                                  ui64, (void**)&pr) || NULL == pr) {
                 pr = OBJ_NEW(mca_oob_tcp_component_peer_t);
@@ -886,10 +930,81 @@ static int component_set_addr(orte_process_name_t *peer,
              */
             opal_output_verbose(OOB_TCP_DEBUG_CONNECT, orte_oob_base_framework.framework_output,
                                 "%s PASSING ADDR %s TO INTERFACE %s AT KERNEL INDEX %d",
-                                ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), addrs[j],
+                                ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), host,
                                 mod->if_name, k);
             mod->api.set_peer((struct mca_oob_tcp_module_t*)mod,
-                              peer, af_family, addrs[j], ports);
+                              peer, af_family, host, ports);
+            found = true;
+            assigned = true;
+        }
+        /* if we cycled thru all their addresses without finding a match in our
+         * interfaces, then it remains possible that they have a routed system
+         * that can still route messages to the destination. So give it a chance
+         * to succeed by assigning our first module to try, and let the normal
+         * failure progression ultimately determine if we can reach this peer
+         */
+        if (!assigned) {
+#if OPAL_ENABLE_IPV6
+            if (AF_INET6 == af_family) {
+                if (NULL == mca_oob_tcp_component.ipv6conns ||
+                    NULL == mca_oob_tcp_component.ipv6conns[0]) {
+                    continue;
+                }
+                host = mca_oob_tcp_component.ipv6conns[0];
+            } else {
+#endif
+                if (NULL == mca_oob_tcp_component.ipv4conns ||
+                    NULL == mca_oob_tcp_component.ipv4conns[0]) {
+                    continue;
+                }
+                host = mca_oob_tcp_component.ipv4conns[0];
+#if OPAL_ENABLE_IPV6
+            }
+#endif
+            /* lookup the kernel index of this address */
+            if (0 >= (k = opal_ifaddrtokindex(host))) {
+                opal_output_verbose(OOB_TCP_DEBUG_CONNECT, orte_oob_base_framework.framework_output,
+                                    "%s UNFOUND KERNEL INDEX %d FOR ADDRESS %s",
+                                    ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), k, host);
+                /* we don't have an interface on this subnet - ignore it */
+                continue;
+            }
+            if (NULL == (mod = (mca_oob_tcp_module_t*)opal_pointer_array_get_item(&mca_oob_tcp_component.modules, k))) {
+                opal_output_verbose(OOB_TCP_DEBUG_CONNECT, orte_oob_base_framework.framework_output,
+                                    "%s NO MODULE AT KINDEX %d FOR ADDRESS %s",
+                                    ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), k, host);
+                continue;
+            }
+            /* record that this peer may be reachable via this module, but don't assign
+             * the peer to this module until later when we actually connect
+             */
+            opal_output_verbose(OOB_TCP_DEBUG_CONNECT, orte_oob_base_framework.framework_output,
+                                "%s PEER %s MAY BE REACHABLE BY ROUTING - ASSIGNING MODULE AT KINDEX %d INTERFACE %s",
+                                ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                                ORTE_NAME_PRINT(peer), k, mod->if_name);
+            if (OPAL_SUCCESS != opal_hash_table_get_value_uint64(&mca_oob_tcp_component.peers,
+                                                                 ui64, (void**)&pr) || NULL == pr) {
+                pr = OBJ_NEW(mca_oob_tcp_component_peer_t);
+                if (OPAL_SUCCESS != (rc = opal_hash_table_set_value_uint64(&mca_oob_tcp_component.peers,
+                                                                           ui64, (void*)pr))) {
+                    ORTE_ERROR_LOG(rc);
+                    return rc;
+                }
+            }
+            opal_bitmap_set_bit(&pr->reachable, k);
+            /* pass this proc, and its ports, to the
+             * module for handling - this module will be responsible
+             * for communicating with the proc via this network.
+             * Note that the modules are *not* necessarily running
+             * on our event base - thus, the modules will push this
+             * call into their own event base for processing.
+             */
+            opal_output_verbose(OOB_TCP_DEBUG_CONNECT, orte_oob_base_framework.framework_output,
+                                "%s PASSING ADDR %s TO INTERFACE %s AT KERNEL INDEX %d",
+                                ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), host,
+                                mod->if_name, k);
+            mod->api.set_peer((struct mca_oob_tcp_module_t*)mod,
+                              peer, af_family, host, ports);
             found = true;
         }
         if (NULL != addrs) {
@@ -1181,6 +1296,9 @@ void mca_oob_tcp_component_no_route(int fd, short args, void *cbdata)
         if (ORTE_SUCCESS != orte_routed.route_lost(&mop->hop)) {
             ORTE_ACTIVATE_PROC_STATE(&mop->hop, ORTE_PROC_STATE_LIFELINE_LOST);
         } else {
+            orte_show_help("help-oob-tcp.txt", "unable-to-commiunicate",
+                           true, ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                           ORTE_NAME_PRINT(&mop->hop));
             ORTE_ACTIVATE_PROC_STATE(&mop->hop, ORTE_PROC_STATE_COMM_FAILED);
         }
     }
@@ -1196,6 +1314,7 @@ void mca_oob_tcp_component_hop_unknown(int fd, short args, void *cbdata)
     uint64_t ui64;
     int k;
     mca_oob_tcp_component_peer_t *pr;
+    orte_rml_send_t *snd;
 
     opal_output_verbose(OOB_TCP_DEBUG_CONNECT, orte_oob_base_framework.framework_output,
                         "%s tcp:unknown hop called for peer %s on interface %s",
@@ -1203,14 +1322,21 @@ void mca_oob_tcp_component_hop_unknown(int fd, short args, void *cbdata)
                         ORTE_NAME_PRINT(&mop->hop),
                         mop->mod->if_name);
 
+    if (orte_finalizing || orte_abnormal_term_ordered) {
+        /* just ignore the problem */
+        OBJ_RELEASE(mop);
+        return;
+    }
+
     /* retrieve the hop's name */
     memcpy(&ui64, (char*)&(mop->hop), sizeof(uint64_t));
 
     /* get the peer object */
     if (OPAL_SUCCESS != opal_hash_table_get_value_uint64(&mca_oob_tcp_component.peers,
                                                          ui64, (void**)&pr) || NULL == pr) {
-        ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
+        /* cleanup */
         goto cleanup;
+
     }
     
     /* ensure we mark that this peer isn't reachable by this module */
@@ -1233,20 +1359,24 @@ void mca_oob_tcp_component_hop_unknown(int fd, short args, void *cbdata)
         }
     }
 
-    /* if we get here, then we have no other modules - so we report
-     * the error back to the OOB and let it try other components
-     * or declare a problem
-     */
-    if (!orte_finalizing && !orte_abnormal_term_ordered) {
-        /* if this was a lifeline, then alert */
-        if (ORTE_SUCCESS != orte_routed.route_lost(&mop->hop)) {
-            ORTE_ACTIVATE_PROC_STATE(&mop->hop, ORTE_PROC_STATE_LIFELINE_LOST);
-        } else {
-            ORTE_ACTIVATE_PROC_STATE(&mop->hop, ORTE_PROC_STATE_COMM_FAILED);
-        }
-    }
-
  cleanup:
+    /* post the message to the OOB so it can see
+     * if another component can transfer it
+     */
+    MCA_OOB_TCP_HDR_NTOH(&mop->snd->hdr);
+    snd = OBJ_NEW(orte_rml_send_t);
+    snd->dst = mop->snd->hdr.dst;
+    snd->origin = mop->snd->hdr.origin;
+    snd->tag = mop->snd->hdr.tag;
+    snd->data = mop->snd->data;
+    snd->count = mop->snd->hdr.nbytes;
+    snd->cbfunc.iov = NULL;
+    snd->cbdata = NULL;
+    /* activate the OOB send state */
+    ORTE_OOB_SEND(snd);
+    /* protect the data */
+    mop->snd->data = NULL;
+
     OBJ_RELEASE(mop);
 }
 
@@ -1446,7 +1576,20 @@ static void peer_cons(mca_oob_tcp_peer_t *peer)
 }
 static void peer_des(mca_oob_tcp_peer_t *peer)
 {
+    if (peer->send_ev_active) {
+        opal_event_del(&peer->send_event);
+    }
+    if (peer->recv_ev_active) {
+        opal_event_del(&peer->recv_event);
+    }
+    if (peer->timer_ev_active) {
+        opal_event_del(&peer->timer_event);
+    }
     if (0 <= peer->sd) {
+        opal_output_verbose(2, orte_oob_base_framework.framework_output,
+                            "%s CLOSING SOCKET %d",
+                            ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
+                            peer->sd);
         CLOSE_THE_SOCKET(peer->sd);
     }
     OPAL_LIST_DESTRUCT(&peer->addrs);
