@@ -14,6 +14,7 @@
  *                         reserved. 
  * Copyright (c) 2008-2009 Sun Microsystems, Inc.  All rights reserved.
  * Copyright (c) 2011      IBM Corporation.  All rights reserved.
+ * Copyright (c) 2014      Intel Corporation.  All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -300,6 +301,10 @@ static void rsh_wait_daemon(pid_t pid, int status, void* cbdata)
             daemon->state = ORTE_PROC_STATE_FAILED_TO_START;
             /* increment the #daemons terminated so we will exit properly */
             jdata->num_terminated++;
+            /* remove it from the routing table to ensure num_routes
+             * returns the correct value
+             */
+            orte_routed.route_lost(&daemon->name);
             /* report that the daemon has failed so we can exit */
             ORTE_ACTIVATE_PROC_STATE(&daemon->name, ORTE_PROC_STATE_FAILED_TO_START);
         }
@@ -1010,9 +1015,7 @@ static void launch_daemons(int fd, short args, void *cbdata)
          * job to move to the following step
          */
         state->jdata->state = ORTE_JOB_STATE_DAEMONS_LAUNCHED;
-        if (ORTE_JOB_STATE_DAEMONS_REPORTED == daemons->state) {
-            ORTE_ACTIVATE_JOB_STATE(state->jdata, ORTE_JOB_STATE_DAEMONS_REPORTED);
-        }
+        ORTE_ACTIVATE_JOB_STATE(state->jdata, ORTE_JOB_STATE_DAEMONS_REPORTED);
         OBJ_RELEASE(state);
         return;
     }
@@ -1244,22 +1247,8 @@ static int rsh_terminate_orteds(void)
 {
     int rc;
     
-    /* now tell them to die */
-    if (orte_abnormal_term_ordered) {
-        /* cannot know if a daemon is able to
-         * tell us it died, so just ensure they
-         * all terminate
-         */
-        if (ORTE_SUCCESS != (rc = orte_plm_base_orted_exit(ORTE_DAEMON_HALT_VM_CMD))) {
-            ORTE_ERROR_LOG(rc);
-        }
-    } else {
-        /* we need them to "phone home", though,
-         * so we can know that they have exited
-         */
-        if (ORTE_SUCCESS != (rc = orte_plm_base_orted_exit(ORTE_DAEMON_EXIT_CMD))) {
-            ORTE_ERROR_LOG(rc);
-        }
+    if (ORTE_SUCCESS != (rc = orte_plm_base_orted_exit(ORTE_DAEMON_EXIT_CMD))) {
+        ORTE_ERROR_LOG(rc);
     }
     
     return rc;
@@ -1492,26 +1481,32 @@ static int setup_shell(orte_plm_rsh_shell_t *rshell,
                        char *nodename, int *argc, char ***argv)
 {
     orte_plm_rsh_shell_t remote_shell, local_shell;
-    struct passwd *p;
     char *param;
     int rc;
-    
+
     /* What is our local shell? */
     local_shell = ORTE_PLM_RSH_SHELL_UNKNOWN;
-    p = getpwuid(getuid());
-    if( NULL == p ) {
-        /* This user is unknown to the system. Therefore, there is no reason we
-         * spawn whatsoever in his name. Give up with a HUGE error message.
-         */
-        orte_show_help( "help-plm-rsh.txt", "unknown-user", true, (int)getuid() );
-        return ORTE_ERR_FATAL;
+
+#if OPAL_ENABLE_GETPWUID
+    {
+        struct passwd *p;
+
+        p = getpwuid(getuid());
+        if( NULL == p ) {
+            /* This user is unknown to the system. Therefore, there is no reason we
+             * spawn whatsoever in his name. Give up with a HUGE error message.
+             */
+            orte_show_help( "help-plm-rsh.txt", "unknown-user", true, (int)getuid() );
+            return ORTE_ERR_FATAL;
+        }
+        param = p->pw_shell;
+        local_shell = find_shell(p->pw_shell);
     }
-    param = p->pw_shell;
-    local_shell = find_shell(p->pw_shell);
-    
+#endif
+
     /* If we didn't find it in getpwuid(), try looking at the $SHELL
-     environment variable (see https://svn.open-mpi.org/trac/ompi/ticket/1060)
-     */
+       environment variable (see https://svn.open-mpi.org/trac/ompi/ticket/1060)
+    */
     if (ORTE_PLM_RSH_SHELL_UNKNOWN == local_shell && 
         NULL != (param = getenv("SHELL"))) {
         local_shell = find_shell(param);
@@ -1554,12 +1549,12 @@ static int setup_shell(orte_plm_rsh_shell_t *rshell,
                          remote_shell, orte_plm_rsh_shell_name[remote_shell]));
     
     /* Do we need to source .profile on the remote side?
-     - sh: yes (see bash(1))
-     - ksh: yes (see ksh(1))
-     - bash: no (see bash(1))
-     - [t]csh: no (see csh(1) and tcsh(1))
-     - zsh: no (see http://zsh.sourceforge.net/FAQ/zshfaq03.html#l19)
-     */
+       - sh: yes (see bash(1))
+       - ksh: yes (see ksh(1))
+       - bash: no (see bash(1))
+       - [t]csh: no (see csh(1) and tcsh(1))
+       - zsh: no (see http://zsh.sourceforge.net/FAQ/zshfaq03.html#l19)
+    */
     
     if (ORTE_PLM_RSH_SHELL_SH == remote_shell ||
         ORTE_PLM_RSH_SHELL_KSH == remote_shell) {
