@@ -9,6 +9,8 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2012-2013 Los Alamos National Security, LLC.
+ *                         All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -23,18 +25,23 @@
 #ifndef ORTE_RAS_SLURM_H
 #define ORTE_RAS_SLURM_H
 
+#include "orte_config.h"
 #include "orte/mca/ras/ras.h"
 #include "orte/mca/ras/base/base.h"
 
-#if defined(c_plusplus) || defined(__cplusplus)
-extern "C" {
-#endif
+BEGIN_C_DECLS
 
-    OMPI_COMP_EXPORT extern orte_ras_base_component_1_0_0_t mca_ras_slurm_component;
-    OMPI_COMP_EXPORT extern orte_ras_base_module_t orte_ras_slurm_module;
+typedef struct {
+    orte_ras_base_component_t super;
+    int timeout;
+    bool dyn_alloc_enabled;
+    char *config_file;
+    bool rolling_alloc;
+} orte_ras_slurm_component_t;
+ORTE_DECLSPEC extern orte_ras_slurm_component_t mca_ras_slurm_component;
 
-#if defined(c_plusplus) || defined(__cplusplus)
-}
-#endif
+ORTE_DECLSPEC extern orte_ras_base_module_t orte_ras_slurm_module;
+
+END_C_DECLS
 
 #endif
