@@ -112,9 +112,10 @@ ORTE_DECLSPEC extern orte_rml_component_t *orte_rml_component;
 /* structure to send RML messages - used internally */
 typedef struct {
     opal_list_item_t super;
-    orte_process_name_t peer;  // targeted recipient
-    int status;                // returned status on send
-    orte_rml_tag_t tag;        // targeted tag
+    orte_process_name_t dst;     // targeted recipient
+    orte_process_name_t origin;
+    int status;                  // returned status on send
+    orte_rml_tag_t tag;          // targeted tag
 
     /* user's send callback functions and data */
     union {
@@ -128,6 +129,10 @@ typedef struct {
     int count;
     /* pointer to the user's buffer */
     opal_buffer_t *buffer;
+    /* pointer to raw data for cross-transport
+     * transfers
+     */
+    char *data;
 } orte_rml_send_t;
 OBJ_CLASS_DECLARATION(orte_rml_send_t);
 
@@ -208,18 +213,18 @@ OBJ_CLASS_DECLARATION(orte_rml_recv_request_t);
         opal_output_verbose(5, orte_rml_base_framework.framework_output, \
                             "%s-%s Send message complete at %s:%d",     \
                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),         \
-                            ORTE_NAME_PRINT(&((m)->peer)),              \
+                            ORTE_NAME_PRINT(&((m)->dst)),               \
                             __FILE__, __LINE__);                        \
         if (NULL != (m)->iov) {                                         \
             if (NULL != (m)->cbfunc.iov) {                              \
                 (m)->cbfunc.iov((m)->status,                            \
-                                &((m)->peer),                           \
+                                &((m)->dst),                            \
                                 (m)->iov, (m)->count,                   \
                                 (m)->tag, (m)->cbdata);                 \
             }                                                           \
         } else {                                                        \
             /* non-blocking buffer send */                              \
-            (m)->cbfunc.buffer((m)->status, &((m)->peer),               \
+            (m)->cbfunc.buffer((m)->status, &((m)->origin),             \
                                (m)->buffer,                             \
                                (m)->tag, (m)->cbdata);                  \
         }                                                               \
