@@ -11,7 +11,8 @@
  *                         All rights reserved.
  * Copyright (c) 2007      Los Alamos National Security, LLC.  All rights
  *                         reserved. 
- * Copyright (c) 2011 Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2011      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2013      Intel, Inc. All rights reserved.
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -40,7 +41,7 @@
 #include "orte/util/proc_info.h"
 #include "orte/util/name_fns.h"
 #include "orte/mca/errmgr/errmgr.h"
-#include "orte/mca/ras/ras_types.h"
+#include "orte/mca/ras/base/base.h"
 #include "orte/runtime/orte_globals.h"
 
 #include "orte/util/hostfile/hostfile_lex.h"
@@ -108,7 +109,7 @@ static char *hostfile_parse_string(void)
     return strdup(orte_util_hostfile_value.sval);
 }
 
-static orte_node_t* hostfile_lookup(opal_list_t* nodes, const char* name)
+static orte_node_t* hostfile_lookup(opal_list_t* nodes, const char* name, bool keep)
 {
     opal_list_item_t* item;
     for(item =  opal_list_get_first(nodes);
@@ -116,18 +117,20 @@ static orte_node_t* hostfile_lookup(opal_list_t* nodes, const char* name)
         item =  opal_list_get_next(item)) {
         orte_node_t* node = (orte_node_t*)item;
         if(strcmp(node->name, name) == 0) {
-            opal_list_remove_item(nodes, item);
+            if (!keep) {
+                opal_list_remove_item(nodes, item);
+            }
             return node;
         }
     }
     return NULL;
 }
 
-static int hostfile_parse_line(int token, opal_list_t* updates, opal_list_t* exclude, bool keep_all)
+static int hostfile_parse_line(int token, opal_list_t* updates,
+                               opal_list_t* exclude, bool keep_all)
 {
     int rc;
     orte_node_t* node;
-    bool got_count = false;
     bool got_max = false;
     char* value;
     char** argv;
@@ -175,7 +178,7 @@ static int hostfile_parse_line(int token, opal_list_t* updates, opal_list_t* exc
             }
             node_name[len-1] = '\0';  /* truncate */
             
-            OPAL_OUTPUT_VERBOSE((3, orte_debug_output,
+            OPAL_OUTPUT_VERBOSE((3, orte_ras_base_framework.framework_output,
                                  "%s hostfile: node %s is being excluded",
                                  ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), node_name));
             
@@ -191,8 +194,8 @@ static int hostfile_parse_line(int token, opal_list_t* updates, opal_list_t* exc
             }
             
             /* Do we need to make a new node object?  First check to see
-             if it's already in the exclude list */
-            if (NULL == (node = hostfile_lookup(exclude, node_name))) {
+               if it's already in the exclude list */
+            if (NULL == (node = hostfile_lookup(exclude, node_name, keep_all))) {
                 node = OBJ_NEW(orte_node_t);
                 node->name = node_name;
                 if (NULL != username) {
@@ -200,9 +203,9 @@ static int hostfile_parse_line(int token, opal_list_t* updates, opal_list_t* exc
                 }
             }
             /* Note that we need to add this back to the exclude list.
-             If it was found, we just removed it (in hostfile_lookup()),
-             so this puts it back. If it was not found, then we have to
-             add it to the exclude list anyway. */
+               If it was found, we just removed it (in hostfile_lookup()),
+               so this puts it back. If it was not found, then we have to
+               add it to the exclude list anyway. */
             opal_list_append(exclude, &node->super);
             return ORTE_SUCCESS;
         }
@@ -220,23 +223,23 @@ static int hostfile_parse_line(int token, opal_list_t* updates, opal_list_t* exc
             node_name = strdup(orte_process_info.nodename);
         }
 
-        OPAL_OUTPUT_VERBOSE((3, orte_debug_output,
+        OPAL_OUTPUT_VERBOSE((3, orte_ras_base_framework.framework_output,
                              "%s hostfile: node %s is being included - keep all is %s",
                              ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), node_name,
                              keep_all ? "TRUE" : "FALSE"));
 
-        /* Do we need to make a new node object?  First check to see
-         * if we are keeping everything or if it's already in the updates
-         * list. Because we check keep_all first, if that is set we will
-         * not do the hostfile_lookup call, and thus won't remove the
-         * pre-existing node from the updates list
-         */
-        if (keep_all || NULL == (node = hostfile_lookup(updates, node_name))) {
+        /* Do we need to make a new node object? */
+        if (NULL == (node = hostfile_lookup(updates, node_name, keep_all))) {
             node = OBJ_NEW(orte_node_t);
             node->name = node_name;
+            node->slots = 1;
             if (NULL != username) {
                 node->username = strdup(username);
             }
+        } else {
+            /* this node was already found once - add a slot and mark slots as "given" */
+            node->slots++;
+            node->slots_given = true;
         }
         /* do we need to record an alias for this node? */
         if (NULL != node_alias) {
@@ -285,27 +288,25 @@ static int hostfile_parse_line(int token, opal_list_t* updates, opal_list_t* exc
             opal_output(0, "WARNING: Unhandled user@host-combination\n"); /* XXX */
         }
         opal_argv_free (argv);
-        /* Do we need to make a new node object?  First check to see
-         * if we are keeping everything or if it's already in the updates
-         * list. Because we check keep_all first, if that is set we will
-         * not do the hostfile_lookup call, and thus won't remove the
-         * pre-existing node from the updates list
-         */
-        if (keep_all || NULL == (node = hostfile_lookup(updates, node_name))) {
+        /* Do we need to make a new node object? */
+        if (NULL == (node = hostfile_lookup(updates, node_name, keep_all))) {
             node = OBJ_NEW(orte_node_t);
             node->name = node_name;
+            node->slots = 1;
             if (NULL != username) {
                 node->username = strdup(username);
             }
+        } else {
+            /* add a slot */
+            node->slots++;
         }
-        /* add a slot */
-        node->slots++;
-        /* do we need to record an alias for this node? */
-        if (NULL != node_alias) {
-            /* add to list of aliases for this node - only add if unique */
-            opal_argv_append_unique_nosize(&node->alias, node_alias, false);
-            free(node_alias);
-        }
+        OPAL_OUTPUT_VERBOSE((1, orte_ras_base_framework.framework_output,
+                             "%s hostfile: node %s slots %d",
+                             ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), node->name, node->slots));
+        /* mark the slots as "given" since we take them as being the
+         * number specified via the rankfile
+         */
+        node->slots_given = true;
         /* skip to end of line */
         while (!orte_util_hostfile_done &&
                ORTE_HOSTFILE_NEWLINE != token) {
@@ -318,7 +319,6 @@ static int hostfile_parse_line(int token, opal_list_t* updates, opal_list_t* exc
         return ORTE_ERROR;
     }
     
-    got_count = false;
     while (!orte_util_hostfile_done) {
         token = orte_util_hostfile_lex();
         
@@ -344,8 +344,18 @@ static int hostfile_parse_line(int token, opal_list_t* updates, opal_list_t* exc
                 OBJ_RELEASE(node);
                 return ORTE_ERROR;
             }
-            node->slots += rc;
-            got_count = true;
+            if (node->slots_given) {
+                /* multiple definitions were given for the
+                 * slot count - this is not allowed
+                 */
+                orte_show_help("help-hostfile.txt", "slots-given",
+                               true,
+                               cur_hostfile_name, node->name);
+                OBJ_RELEASE(node);
+                return ORTE_ERROR;
+            }
+            node->slots = rc;
+            node->slots_given = true;
 
             /* Ensure that slots_max >= slots */
             if (node->slots_max != 0 && node->slots_max < node->slots) {
@@ -390,13 +400,10 @@ static int hostfile_parse_line(int token, opal_list_t* updates, opal_list_t* exc
         }
     }
 
-done:
-    if (!got_count) {
-        if (got_max) {
-            node->slots = node->slots_max;
-        } else {
-            ++node->slots;
-        }
+ done:
+    if (got_max && !node->slots_given) {
+        node->slots = node->slots_max;
+        node->slots_given = true;
     }
     opal_list_append(updates, &node->super);
 
@@ -408,7 +415,8 @@ done:
  * Parse the specified file into a node list.
  */
 
-static int hostfile_parse(const char *hostfile, opal_list_t* updates, opal_list_t* exclude, bool keep_all)
+static int hostfile_parse(const char *hostfile, opal_list_t* updates,
+                          opal_list_t* exclude, bool keep_all)
 {
     int token;
     int rc = ORTE_SUCCESS;
@@ -493,27 +501,29 @@ unlock:
 int orte_util_add_hostfile_nodes(opal_list_t *nodes,
                                  char *hostfile)
 {
-    opal_list_t exclude;
+    opal_list_t exclude, adds;
     opal_list_item_t *item, *itm;
     int rc;
+    orte_node_t *nd, *node;
+    bool found;
 
-    
-    OPAL_OUTPUT_VERBOSE((1, orte_debug_output,
+    OPAL_OUTPUT_VERBOSE((1, orte_ras_base_framework.framework_output,
                          "%s hostfile: checking hostfile %s for nodes",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), hostfile));
 
     OBJ_CONSTRUCT(&exclude, opal_list_t);
-    
-    /* parse the hostfile and add the contents to the list */
-    if (ORTE_SUCCESS != (rc = hostfile_parse(hostfile, nodes, &exclude, false))) {
+    OBJ_CONSTRUCT(&adds, opal_list_t);
+
+    /* parse the hostfile and add any new contents to the list */
+    if (ORTE_SUCCESS != (rc = hostfile_parse(hostfile, &adds, &exclude, false))) {
         goto cleanup;
     }
     
-    /* parse the nodes to check for any relative node directives */
-    for (item = opal_list_get_first(nodes);
-         item != opal_list_get_end(nodes);
+    /* check for any relative node directives */
+    for (item = opal_list_get_first(&adds);
+         item != opal_list_get_end(&adds);
          item = opal_list_get_next(item)) {
-        orte_node_t *node=(orte_node_t*)item;
+        node=(orte_node_t*)item;
         
         if ('+' == node->name[0]) {
             orte_show_help("help-hostfile.txt", "hostfile:relative-syntax",
@@ -524,16 +534,16 @@ int orte_util_add_hostfile_nodes(opal_list_t *nodes,
     }
     
     /* remove from the list of nodes those that are in the exclude list */
-    while(NULL != (item = opal_list_remove_first(&exclude))) {
-        orte_node_t *exnode = (orte_node_t*)item;
+    while (NULL != (item = opal_list_remove_first(&exclude))) {
+        nd = (orte_node_t*)item;
         /* check for matches on nodes */
-        for (itm = opal_list_get_first(nodes);
-             itm != opal_list_get_end(nodes);
+        for (itm = opal_list_get_first(&adds);
+             itm != opal_list_get_end(&adds);
              itm = opal_list_get_next(itm)) {
-            orte_node_t *node=(orte_node_t*)itm;
-            if (0 == strcmp(exnode->name, node->name)) {
+            node = (orte_node_t*)itm;
+            if (0 == strcmp(nd->name, node->name)) {
                 /* match - remove it */
-                opal_list_remove_item(nodes, itm);
+                opal_list_remove_item(&adds, itm);
                 OBJ_RELEASE(itm);
                 break;
             }
@@ -541,8 +551,32 @@ int orte_util_add_hostfile_nodes(opal_list_t *nodes,
         OBJ_RELEASE(item);
     }
     
+    /* transfer across all unique nodes */
+    while (NULL != (item = opal_list_remove_first(&adds))) {
+        nd = (orte_node_t*)item;
+        found = false;
+        for (itm = opal_list_get_first(nodes);
+             itm != opal_list_get_end(nodes);
+             itm = opal_list_get_next(itm)) {
+            node = (orte_node_t*)itm;
+            if (0 == strcmp(nd->name, node->name)) {
+                found = true;
+                break;
+            }
+        }
+        if (!found) {
+            opal_list_append(nodes, &nd->super);
+            OPAL_OUTPUT_VERBOSE((1, orte_ras_base_framework.framework_output,
+                                 "%s hostfile: adding node %s slots %d",
+                                 ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), nd->name, nd->slots));
+        } else {
+            OBJ_RELEASE(item);
+        }
+    }
+
 cleanup:
-    OBJ_DESTRUCT(&exclude);
+    OPAL_LIST_DESTRUCT(&exclude);
+    OPAL_LIST_DESTRUCT(&adds);
 
     return rc;
 }
@@ -565,7 +599,7 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
     opal_list_t keep;
     bool found;
 
-    OPAL_OUTPUT_VERBOSE((1, orte_debug_output,
+    OPAL_OUTPUT_VERBOSE((1, orte_ras_base_framework.framework_output,
                         "%s hostfile: filtering nodes through hostfile %s",
                         ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), hostfile));
 
@@ -597,6 +631,7 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
             orte_node_t *node = (orte_node_t*)item2;
             if (0 == strcmp(node_from_file->name, node->name)) {
                 /* match - remove it */
+                opal_output(0, "HOST %s ON EXCLUDE LIST - REMOVING", node->name);
                 opal_list_remove_item(&newnodes, item2);
                 OBJ_RELEASE(item2);
                 break;
@@ -725,6 +760,8 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
                  * we have to check for local interfaces
                  */
                 if (0 == strcmp(node_from_file->name, node_from_list->name) ||
+                    (0 == strcmp(node_from_file->name, "localhost") &&
+                     0 == strcmp(node_from_list->name, orte_process_info.nodename)) ||
                     (opal_ifislocal(node_from_list->name) &&
                      opal_ifislocal(node_from_file->name))) {
                     /* if the slot count here is less than the
@@ -733,7 +770,7 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
                      * to subdivide an allocation
                      */
                     if (node_from_file->slots < node_from_list->slots) {
-                        node_from_list->slots_alloc = node_from_file->slots;
+                        node_from_list->slots = node_from_file->slots;
                     }
                     if (remove) {
                         /* remove the node from the list */
@@ -748,16 +785,16 @@ int orte_util_filter_hostfile_nodes(opal_list_t *nodes,
                     break;
                 }
             }
-            /* if the host in the newnode list wasn't found, 
-             * then that is an error we need to report to the 
-             * user and abort 
-             */ 
-            if (!found) { 
-                orte_show_help("help-hostfile.txt", "hostfile:extra-node-not-found", 
-                               true, hostfile, node_from_file->name); 
-                rc = ORTE_ERR_SILENT; 
-                goto cleanup; 
-            } 
+            /* if the host in the newnode list wasn't found,
+             * then that is an error we need to report to the
+             * user and abort
+             */
+            if (!found) {
+                orte_show_help("help-hostfile.txt", "hostfile:extra-node-not-found",
+                               true, hostfile, node_from_file->name);
+                rc = ORTE_ERR_SILENT;
+                goto cleanup;
+            }
         }
         /* cleanup the newnode list */
         OBJ_RELEASE(item2);
@@ -810,7 +847,7 @@ int orte_util_get_ordered_host_list(opal_list_t *nodes,
     orte_node_t *node_from_pool, *newnode;
     int rc;
     
-    OPAL_OUTPUT_VERBOSE((1, orte_debug_output,
+    OPAL_OUTPUT_VERBOSE((1, orte_ras_base_framework.framework_output,
                          "%s hostfile: creating ordered list of hosts from hostfile %s",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME), hostfile));
     
@@ -868,9 +905,9 @@ int orte_util_get_ordered_host_list(opal_list_t *nodes,
                      * to subdivide an allocation
                      */
                     if (node->slots < node_from_pool->slots) {
-                        newnode->slots_alloc = node->slots;
+                        newnode->slots = node->slots;
                     } else {
-                        newnode->slots_alloc = node_from_pool->slots;
+                        newnode->slots = node_from_pool->slots;
                     }
                     opal_list_insert_pos(nodes, item1, &newnode->super);
                     /* track number added */
@@ -921,9 +958,9 @@ int orte_util_get_ordered_host_list(opal_list_t *nodes,
              * to subdivide an allocation
              */
             if (node->slots < node_from_pool->slots) {
-                newnode->slots_alloc = node->slots;
+                newnode->slots = node->slots;
             } else {
-                newnode->slots_alloc = node_from_pool->slots;
+                newnode->slots = node_from_pool->slots;
             }
             /* insert it before item1 */
             opal_list_insert_pos(nodes, item1, &newnode->super);
